@@ -3,7 +3,7 @@
  * Mark Nagel <nagel@ics.uci.edu>
  * 20 July 1989
  *
- * $Revision: 7.2 $
+ * $Revision: 7.7 $
  *
  * bool
  * format(fmt, num, buf, buflen)
@@ -107,178 +107,160 @@ static void reverse(register char *buf);
 bool
 format(char *fmt, double val, char *buf, int buflen)
 {
-  register char *cp;
-  char *tmp, *tp;
-  bool comma = false, negative = false;
-  char *integer = NULL, *decimal = NULL;
-  char *exponent = NULL;
-  int exp_val, width;
-  char prtfmt[32];
-  static char		*mantissa = NULL;
-  static char		*tmpfmt1 = NULL, *tmpfmt2 = NULL, *exptmp = NULL;
-  static unsigned	mantlen = 0, fmtlen = 0;
-  char *fraction = NULL;
-  int zero_pad = 0;
+    register char *cp;
+    char *tmp, *tp;
+    bool comma = false, negative = false;
+    char *integer = NULL, *decimal = NULL;
+    char *exponent = NULL;
+    int exp_val = 0;
+    int width;
+    char prtfmt[32];
+    static char		*mantissa = NULL;
+    static char		*tmpfmt1 = NULL, *tmpfmt2 = NULL, *exptmp = NULL;
+    static unsigned	mantlen = 0, fmtlen = 0;
+    char *fraction = NULL;
+    int zero_pad = 0;
 
-  if (fmt == NULL)
-    return(true);
+    if (fmt == NULL)
+	return(true);
 
-  if (strlen(fmt) + 1 > fmtlen)
-  {	fmtlen = strlen(fmt) + 40;
+    if (strlen(fmt) + 1 > fmtlen) {
+	fmtlen = strlen(fmt) + 40;
 	tmpfmt1 = scxrealloc(tmpfmt1, fmtlen);
 	tmpfmt2 = scxrealloc(tmpfmt2, fmtlen);
 	exptmp = scxrealloc(exptmp, fmtlen);
-  }
-  fmt = strcpy(tmpfmt1, fmt);
-  if (buflen + 1 > mantlen)
-  {	mantlen = buflen + 40;
+    }
+    fmt = strcpy(tmpfmt1, fmt);
+    if (buflen + 1 > mantlen) {
+    	mantlen = buflen + 40;
 	mantissa = scxrealloc(mantissa, mantlen);
-  }
+    }
 
-  /*
-   * select positive or negative format if necessary
-   */
-  for (cp = fmt; *cp != ';' && *cp != EOS; cp++)
-  {
+/*
+ * select positive or negative format if necessary
+ */
+    for (cp = fmt; *cp != ';' && *cp != EOS; cp++) {
     if (*cp == '\\')
-      cp++;
-  }
-  if (*cp == ';')
-  {
-    if (val < 0.0)
-    {
-      val = -val;     /* format should provide sign if desired */
-      fmt = cp + 1;
+	cp++;
     }
-    else
-    {
-      *cp = EOS;
+    if (*cp == ';') {
+	if (val < 0.0) {
+	    val = -val;     /* format should provide sign if desired */
+	    fmt = cp + 1;
+	} else
+	    *cp = EOS;
     }
-  }
   
-  /*
-   * extract other information from format and produce a
-   * format string stored in tmpfmt2 also scxmalloc()'d above
-   */
-  tmp = tmpfmt2;
-  for (cp = fmt, tp = tmp; *cp != EOS; cp++)
-  {
-    switch (*cp)
-    {
-      case '\\':
-        *tp++ = *cp++;
-        *tp++ = *cp;
-	break;
+/*
+ * extract other information from format and produce a
+ * format string stored in tmpfmt2 also scxmalloc()'d above
+ */
+    tmp = tmpfmt2;
+    for (cp = fmt, tp = tmp; *cp != EOS; cp++) {
+	switch (*cp) {
+	    case '\\':
+		*tp++ = *cp++;
+		*tp++ = *cp;
+		break;
 
-      case ',':
-        comma = true;
-	break;
+	    case ',':
+		comma = true;
+		break;
 
-      case '.':
-        if (decimal == NULL)
-	  decimal = tp;
-	*tp++ = *cp;
-	break;
+	    case '.':
+		if (decimal == NULL)
+		    decimal = tp;
+		*tp++ = *cp;
+		break;
 	
-      case '%':
-        val *= 100.0;
-	*tp++ = *cp;
-	break;
+	    case '%':
+		val *= 100.0;
+		*tp++ = *cp;
+		break;
 	
-      default:
-        *tp++ = *cp;
-	break;
-    }
-  }
-  *tp = EOS;
-  fmt = tmpfmt2;
-
-  if (val < 0.0)
-  {	negative = true;
-	val = -val;
-  }
-  /*
-   * extract the exponent from the format if present
-   */
-  for (cp = fmt; *cp != EOS; cp++)
-  { if (*cp == '\\')
-    {
-      cp++;
-    }
-    else if (*cp == 'e' || *cp == 'E')
-    {
-      if (cp[1] == '+' || cp[1] == '-')
-      {
-	exponent = strcpy(exptmp, cp);
-	*cp = EOS;
-	exp_val = 0;
-	if (val!=0.0) {
-	  while (val < 1.0)
-	  {
-	    val *= 10.0;
-	    exp_val--;
-	  }
-	  while (val >= 10.0)
-	  {
-	    val /= 10.0;
-	    exp_val++;
-	  }
+	    default:
+		*tp++ = *cp;
+		break;
 	}
-	break;
-      }
     }
-  }
+    *tp = EOS;
+    fmt = tmpfmt2;
 
-  /*
-   * determine maximum decimal places and use sprintf
-   * to build initial character form of formatted value.
-   */
-  width = 0;
-  if (decimal)
-  {
-    *decimal++ = EOS;
-    for (cp = decimal; *cp != EOS; cp++)
+    if (val < 0.0) {
+  	negative = true;
+	val = -val;
+    }
+/*
+ * extract the exponent from the format if present
+ */
+    for (cp = fmt; *cp != EOS; cp++) {
+	if (*cp == '\\')
+	    cp++;
+	else if (*cp == 'e' || *cp == 'E') {
+	    if (cp[1] == '+' || cp[1] == '-') {
+		exponent = strcpy(exptmp, cp);
+		*cp = EOS;
+		if (val != 0.0) {
+		    while (val < 1.0) {
+			val *= 10.0;
+			exp_val--;
+		    }
+		    while (val >= 10.0) {
+			val /= 10.0;
+			exp_val++;
+		    }
+		}
+		break;
+	    }
+	}
+    }
+
+/*
+ * determine maximum decimal places and use sprintf
+ * to build initial character form of formatted value.
+ */
+    width = 0;
+    if (decimal) {
+	*decimal++ = EOS;
+	for (cp = decimal; *cp != EOS; cp++) {
+	    switch (*cp) {
+		case '\\':
+		    cp++;
+		    break;
+
+		case '#':
+		    width++;
+		    break;
+
+		case '0':
+		    zero_pad = ++width;
+		    break;
+	    }
+	}
+	zero_pad = strlen(decimal) - zero_pad;
+    }
+    (void) sprintf(prtfmt, "%%.%dlf", width);
+    (void) sprintf(mantissa, prtfmt, val);
+    for (cp = integer = mantissa; *cp != '.' && *cp != EOS; cp++) {
+	if (*integer == '0')
+	    integer++;
+    }
+    if (*cp == '.') {
+	fraction = cp + 1;
+	*cp = EOS;
+	cp = fraction + strlen(fraction) - 1;
+	for (; zero_pad > 0; zero_pad--, cp--) {
+	    if (*cp == '0')
+		*cp = EOS;
+	    else
+		break;
+	}
+    }
+
+/*
+ * format the puppy
+ */
     {
-      switch (*cp)
-      {
-        case '\\':
-          cp++;
-	  break;
-
-        case '#':
-          width++;
-	  break;
-
-	case '0':
-	  zero_pad = ++width;
-	  break;
-      }
-    }
-    zero_pad = strlen(decimal) - zero_pad;
-  }
-  (void) sprintf(prtfmt, "%%.%dlf", width);
-  (void) sprintf(mantissa, prtfmt, val);
-  for (cp = integer = mantissa; *cp != '.' && *cp != EOS; cp++)
-  {
-    if (*integer == '0')
-      integer++;
-  }
-  if (*cp == '.')
-  {
-    fraction = cp + 1;
-    *cp = EOS;
-    cp = fraction + strlen(fraction) - 1;
-    for (; zero_pad > 0; zero_pad--, cp--)
-    {
-      if (*cp == '0')
-        *cp = EOS;
-    }
-  }
-
-  /*
-   * format the puppy
-   */
-  {
     static	char *citmp = NULL, *cftmp = NULL;
     static	unsigned cilen = 0, cflen = 0;
     char *ci, *cf, *ce;
@@ -287,16 +269,16 @@ format(char *fmt, double val, char *buf, int buflen)
     
     ci = fmt_int(integer, fmt, comma, negative);
     len_ci = strlen(ci);
-    if (len_ci >= cilen)
-    {	cilen = len_ci + 40;
+    if (len_ci >= cilen) {
+    	cilen = len_ci + 40;
 	citmp = scxrealloc(citmp, cilen);
     }
     ci = strcpy(citmp, ci);
 
     cf = (fraction) ? fmt_frac(fraction, decimal) : "";
     len_cf = strlen(cf);
-    if (len_cf >= cflen)
-    {	cflen = len_cf + 40;
+    if (len_cf >= cflen) {
+    	cflen = len_cf + 40;
 	cftmp = scxrealloc(cftmp, cilen);
     }
     cf = strcpy(cftmp, cf);
@@ -307,147 +289,128 @@ format(char *fmt, double val, char *buf, int buflen)
  * Skip copy assuming sprintf doesn't call our format functions
  *   ce = strcpy(scxmalloc((unsigned)((len_ce = strlen(ce)) + 1)), ce);
  */
-    if (len_ci + len_cf + len_ce < buflen)
-    {
-      (void) sprintf(buf, "%s%s%s", ci, cf, ce);
-      ret = true;
+    if (len_ci + len_cf + len_ce < buflen) {
+	(void) sprintf(buf, "%s%s%s", ci, cf, ce);
+	ret = true;
     }
 
     return (ret);
-  }
+    }
 }
 
 /*****************************************************************************/
 
 static char *
 fmt_int(char *val,	/* integer part of the value to be formatted */
-  char *fmt,		/* integer part of the format */
-  bool comma,		/* true if we should comma-ify the value */
-  bool negative)	/* true if the value is actually negative */
+    char *fmt,		/* integer part of the format */
+    bool comma,		/* true if we should comma-ify the value */
+    bool negative)	/* true if the value is actually negative */
 {
-  int digit, f, v;
-  int thousands = 0;
-  char *cp;
-  static char buf[MAXBUF];
-  char *bufptr = buf;
+    int digit, f, v;
+    int thousands = 0;
+    char *cp;
+    static char buf[MAXBUF];
+    char *bufptr = buf;
 
-  /*
-   * locate the leftmost digit placeholder
-   */
-  for (cp = fmt; *cp != EOS; cp++)
-  {
+/*
+ * locate the leftmost digit placeholder
+ */
+    for (cp = fmt; *cp != EOS; cp++) {
     if (*cp == '\\')
-      cp++;
+	cp++;
     else if (*cp == '#' || *cp == '0')
-      break;
-  }
-  digit = (*cp == EOS) ? -1 : cp - fmt;
-
-  /*
-   * format the value
-   */
-  f = strlen(fmt) - 1;
-  v = (digit >= 0) ? strlen(val) - 1 : -1;
-  while (f >= 0 || v >= 0)
-  {
-    if (f > 0 && fmt[f-1] == '\\')
-    {
-      *bufptr++ = fmt[f--];
+	break;
     }
-    else if (f >= 0 && (fmt[f] == '#' || fmt[f] == '0'))
-    {
-      if (v >= 0 || fmt[f] == '0')
-      {
-        *bufptr++ = v < 0 ? '0' : val[v];
-	if (comma && (thousands = (thousands + 1) % 3) == 0 && v > 0)
-	{
-	  *bufptr++ = ',';
+    digit = (*cp == EOS) ? -1 : cp - fmt;
+
+/*
+ * format the value
+ */
+    f = strlen(fmt) - 1;
+    v = (digit >= 0) ? strlen(val) - 1 : -1;
+    while (f >= 0 || v >= 0) {
+	if (f > 0 && fmt[f-1] == '\\') {
+	    *bufptr++ = fmt[f--];
+	} else if (f >= 0 && (fmt[f] == '#' || fmt[f] == '0')) {
+	    if (v >= 0 || fmt[f] == '0') {
+		*bufptr++ = v < 0 ? '0' : val[v];
+		if (comma && (thousands = (thousands + 1) % 3) == 0 && v > 0)
+		    *bufptr++ = ',';
+		v--;
+	    }
+	} else if (f >= 0) {
+	    *bufptr++ = fmt[f];
 	}
-	v--;
-      }
+	if (v >= 0 && f == digit) {
+	    continue;
+	}
+	f--;
     }
-    else if (f >= 0)
-    {
-      *bufptr++ = fmt[f];
-    }
-    if (v >= 0 && f == digit)
-    {
-      continue;
-    }
-    f--;
-  }
     
-  if (negative && digit >= 0)
-    *bufptr++ = '-';
-  *bufptr = EOS;
-  reverse(buf);
+    if (negative && digit >= 0)
+	*bufptr++ = '-';
+    *bufptr = EOS;
+    reverse(buf);
 
-  return (buf);
+    return (buf);
 }
 
 /*****************************************************************************/
 
 static char *
 fmt_frac(char *val,	/* fractional part of the value to be formatted */
-  char *fmt)		/* fractional portion of format */
+    char *fmt)		/* fractional portion of format */
 {
-  static char buf[MAXBUF];
-  register char *bufptr = buf;
-  register char *fmtptr = fmt, *valptr = val;
+    static char buf[MAXBUF];
+    register char *bufptr = buf;
+    register char *fmtptr = fmt, *valptr = val;
 
-  *bufptr++ = '.';
-  while (*fmtptr != EOS)
-  {
-    if (*fmtptr == '\\')
-    {
-      *bufptr++ = *++fmtptr;
+    *bufptr++ = '.';
+    while (*fmtptr != EOS) {
+	if (*fmtptr == '\\')
+	    *bufptr++ = *++fmtptr;
+	else if (*fmtptr == '#' || *fmtptr == '0') {
+	    if (*valptr != EOS || *fmtptr == '0')
+		*bufptr++ = (*valptr != EOS) ? *valptr++ : *fmtptr;
+	} else
+	    *bufptr++ = *fmtptr;
+	fmtptr++;
     }
-    else if (*fmtptr == '#' || *fmtptr == '0')
-    {
-      if (*valptr != EOS || *fmtptr == '0')
-      {
-        *bufptr++ = (*valptr != EOS) ? *valptr++ : *fmtptr;
-      }
-    }
-    else
-    {
-      *bufptr++ = *fmtptr;
-    }
-    fmtptr++;
-  }
-  *bufptr = EOS;
+    *bufptr = EOS;
 
-  return (buf);
+    if (buf[1] == EOS)
+	buf[0] = EOS;
+
+    return (buf);
 }
 
 /*****************************************************************************/
 
 static char *
 fmt_exp(int val,	/* value of the exponent */
-  char *fmt)		/* exponent part of the format */
+    char *fmt)		/* exponent part of the format */
 {
-  static char buf[MAXBUF];
-  register char *bufptr = buf;
-  char valbuf[64];
-  bool negative = false;
+    static char buf[MAXBUF];
+    register char *bufptr = buf;
+    char valbuf[64];
+    bool negative = false;
   
-  *bufptr++ = *fmt++;
-  if (*fmt == '+')
-    *bufptr++ = (val < 0) ? '-' : '+';
-  else if (val < 0)
-    *bufptr++ = '-';
-  fmt++;
-  *bufptr = EOS;
+    *bufptr++ = *fmt++;
+    if (*fmt == '+')
+	*bufptr++ = (val < 0) ? '-' : '+';
+    else if (val < 0)
+	*bufptr++ = '-';
+    fmt++;
+    *bufptr = EOS;
 
-  if (val < 0)
-  {
-    val = -val;
-    negative = false;
-  }
-  (void) sprintf(valbuf, "%d", val);
+    if (val < 0) {
+	val = -val;
+	negative = false;
+    }
+    (void) sprintf(valbuf, "%d", val);
   
-  (void) strcat(buf, fmt_int(valbuf, fmt, false, negative));
-  return (buf);
+    (void) strcat(buf, fmt_int(valbuf, fmt, false, negative));
+    return (buf);
 }
 
 /*****************************************************************************/
@@ -455,15 +418,14 @@ fmt_exp(int val,	/* value of the exponent */
 static void
 reverse(register char *buf)
 {
-  register char *cp = buf + strlen(buf) - 1;
-  register char tmp;
+    register char *cp = buf + strlen(buf) - 1;
+    register char tmp;
 
-  while (buf < cp)
-  {
-    tmp = *cp;
-    *cp-- = *buf;
-    *buf++ = tmp;
-  }
+    while (buf < cp) {
+	tmp = *cp;
+	*cp-- = *buf;
+	*buf++ = tmp;
+    }
 }
 
 /*****************************************************************************/
