@@ -2,13 +2,13 @@
  * Help functions for sc 
  * R. Bond, 1988
  * J. Buhrt 1990
- * $Revision: 7.2 $
+ * $Revision: 7.7 $
  */
 
 #ifdef QREF
 #include <stdio.h>
 char	*header = " Quick Reference";
-char	*revision = "$Revision: 7.2 $";
+char	*revision = "$Revision: 7.7 $";
 #else
 #include <curses.h>
 #include "sc.h"
@@ -185,18 +185,22 @@ char *vi[] = {
 "     h l          Move cursor forward, backward.",
 "     0 $          Move cursor to the beginning, end of the line.",
 "     b w          Move cursor forward/back one word.",
+"     e            Move cursor to next end-of-word.",
 "     fc           Move cursor to character c.",
-"     tc           Move the cursor the the character before c.",
+"     tc           Move the cursor to the character before c.",
 "     i a          Enter insert mode before/after the cursor.",
 "     I A          Move to column 0/end of line and enter insert mode.",
 "     x X          Delete the character under/before the cursor.",
 "     rc           Replace the character under the cursor with c.",
-"     cm           Change - m = b,f,h,l,t or w.",
-"     dm           Delete - m = b,f,h,l,t or w.",
+"     cm dm        Change/Delete - m = b,e,f,h,l,t or w.",
 "     R            Enter replace (overstrike) mode.",
+"     s            Delete character under cursor and enter insert mode.",
 "     + j - k /    Forward/backward/search the command history.",
 "     n            Repeat last history search.",
 "     . u          Repeat/undo the last command.",
+"     ^V           Enter navigate mode.  Another ^V enters current cell address.",
+"     ^W           Type, in the command line, the current cell's expression.",
+"     ^A           Type, in the command line, the current cell's numeric value.",
 (char *)0
 };
 
@@ -322,9 +326,6 @@ char *misc[] = {
 "     C        Redraw the screen with the row containing the current cell"
 "              centered.",
 "     @        Recalculate the spreadsheet.",
-"     ^V       Type, in the command line, the name of the current cell.",
-"     ^W       Type, in the command line, the current cell's expression.",
-"     ^A       Type, in the command line, the current cell's numeric value.",
 "     TAB      When the character cursor is on the top line TAB can be used",
 "              to start or stop the display of the default range.",
 (char *)0
