@@ -7,7 +7,7 @@
  *
  *              More mods Robert Bond, 12/86
  *		More mods by Alan Silverstein, 3/88, see list of changes.
- *		$Revision: 7.2 $
+ *		$Revision: 7.7 $
  *
  */
 
@@ -63,7 +63,7 @@ extern int VMS_read_raw;   /*sigh*/
 extern YYSTYPE yylval;
 #endif /* hpux */
 
-char *strtof(register char *p, double *res);
+char *strtof(char *p, double *res);
 
 jmp_buf wakeup;
 jmp_buf fpe_buf;
@@ -74,14 +74,14 @@ void
 fpe_trap(int signo)
 {
 #if defined(i386) && !defined(M_XENIX)
-	asm("	fnclex");
-	asm("	fwait");
+    asm("	fnclex");
+    asm("	fwait");
 #else
 #ifdef IEEE_MATH
-	(void)fpsetsticky((fp_except)0);	/* Clear exception */
+    (void)fpsetsticky((fp_except)0);	/* Clear exception */
 #endif /* IEEE_MATH */
 #ifdef PC
-	_fpreset();
+    _fpreset();
 #endif
 #endif
     longjmp(fpe_buf, 1);
@@ -94,58 +94,78 @@ struct key {
 
 struct key experres[] = {
 #include "experres.h"
-    0, 0
+    { 0, 0 }
 };
 
 struct key statres[] = {
 #include "statres.h"
-    0, 0
+    { 0, 0 }
 };
 
 int
 yylex()
 {
-    register char *p = line+linelim;
+    char *p = line+linelim;
     int ret;
-    static bool isfunc;
+    static bool isfunc = 0;
+    static bool colstate = 0;
     static int dateflag;
 
     while (isspace(*p)) p++;
-    if (*p == '\0') ret = -1;
-    else if (isalpha(*p)) {
+    if (*p == '\0') {
+	isfunc = 0;
+	ret = -1;
+    } else if (isalpha(*p) || (*p == '_')) {
 	char *tokenst = p;
-	register tokenl;
+	register char *la;	/* lookahead pointer */
+	int tokenl;
 	register struct key *tblp;
 	tokenl = 0;
 	/*
 	 *  This picks up either 1 or 2 alpha characters (a column) or
-	 *  tokens with at least three leading alphas and '_' or digits
-	 *  (a function or token or command or a range name)
+	 *  tokens made up of alphanumeric chars and '_' (a function or
+	 *  token or command or a range name)
 	 */
-	while (isalpha(*p) || ((*p == '_') || (isdigit(*p)) && (tokenl > 2))) {
+	while (isalpha(*p)) {
 	    p++;
 	    tokenl++;
 	}
-	if (tokenl <= 2 && !isfunc) { /* a COL is 1 or 2 char alpha */
+	la = p;
+	while (isdigit(*la) || (*la == '$'))
+	    la++;
+	/*
+	 * A COL is 1 or 2 char alpha with nothing but digits following
+	 * (no alpha or '_')
+	 */
+	if (tokenl && tokenl <= 2 && isfunc < 1 && (colstate ||
+		(isdigit(*(la-1)) && !(isalpha(*la) || (*la == '_'))))) {
 	    ret = COL;
 	    yylval.ival = atocol(tokenst, tokenl);
 	} else {
+	    while (isalpha(*p) || (*p == '_') || isdigit(*p)) {
+		p++;
+		tokenl++;
+	    }
 	    ret = WORD;
-	    for (tblp = linelim ? experres : statres; tblp->key; tblp++)
+	    if (!linelim || isfunc) {
+		if (isfunc) isfunc--;
+		for (tblp = linelim ? experres : statres; tblp->key; tblp++)
 		    if (((tblp->key[0]^tokenst[0])&0137)==0
-		     && tblp->key[tokenl]==0) {
-			register i = 1;
+			    && tblp->key[tokenl]==0) {
+			int i = 1;
 			while (i<tokenl && ((tokenst[i]^tblp->key[i])&0137)==0)
 			    i++;
-			if (i>=tokenl) {
+			if (i >= tokenl) {
 			    ret = tblp->val;
+			    colstate = (ret <= S_FORMAT);
 			    break;
 			}
 		    }
-	    if (ret==WORD) { 
+	    }
+	    if (ret == WORD) {
 		struct range *r;
-		if (r = find_range(tokenst, tokenl,
-				   (struct ent *)0, (struct ent *)0)) {
+		if (!find_range(tokenst, tokenl,
+			(struct ent *)0, (struct ent *)0, &r)) {
 		    yylval.rval.left = r->r_left;
 		    yylval.rval.right = r->r_right;
 		    if (r->r_is_range)
@@ -226,26 +246,26 @@ yylex()
 	char *ptr;
         ptr = p+1;	/* "string" or "string\"quoted\"" */
         while(*ptr && ((*ptr != '"') || (*(ptr-1) == '\\')))
-		ptr++;
+	    ptr++;
         ptr = scxmalloc((unsigned)(ptr-p));
 	yylval.sval = ptr;
-	p += 1;
+	p++;
 	while (*p && ((*p != '"') || (*(p-1) == '\\')))
-		*ptr++ = *p++;
-	*ptr = 0;
+	    *ptr++ = *p++;
+	*ptr = '\0';
 	if (*p)
-		p += 1;
+	    p++;
 	ret = STRING;
     } else if (*p=='[') {
 	while (*p && *p!=']')
-		p++;
+	    p++;
 	if (*p)
-		p++;
+	    p++;
 	linelim = p-line;
 	return yylex();
     } else ret = *p++;
     linelim = p-line;
-    isfunc = ret=='@';
+    if (!isfunc) isfunc = ((ret == '@') - (ret == S_SET || ret == S_GOTO));
     return ret;
 }
 
@@ -262,14 +282,14 @@ yylex()
 int
 atocol(char *string, int len)
 {
-	register int col;
+    register int col;
 
-	col = (string [0] & 0137) - 'A';
+    col = (string[0] & 0137) - 'A';
 
-	if (len == 2)		/* has second char */
-	    col = ((col + 1) * 26) + ((string [1] & 0137) - 'A');
+    if (len == 2)		/* has second char */
+	col = ((col + 1) * 26) + ((string[1] & 0137) - 'A');
 
-	return (col);
+    return (col);
 }
 
 
@@ -292,7 +312,7 @@ resetkbd()
 int
 nmgetch()
 {
-    return (toascii(getchar()));
+    return (getchar());
 }
 
 #else /* VMS */
@@ -318,16 +338,15 @@ nmgetch()
 
     if (VMS_read_raw) {
       VMScheck(smg$read_keystroke (&stdkb->_id, &c, 0, 0, 0));
-    }
-    else
+    } else
        c = getchar();
 
     switch (c) {
-    case SMG$K_TRM_LEFT:  c = ctl('b'); break;
-    case SMG$K_TRM_RIGHT: c = ctl('f'); break;
-    case SMG$K_TRM_UP:    c = ctl('p'); break;
-    case SMG$K_TRM_DOWN:  c = ctl('n'); break;
-    default:   c = c & A_CHARTEXT;
+	case SMG$K_TRM_LEFT:  c = ctl('b'); break;
+	case SMG$K_TRM_RIGHT: c = ctl('f'); break;
+	case SMG$K_TRM_UP:    c = ctl('p'); break;
+	case SMG$K_TRM_DOWN:  c = ctl('n'); break;
+	default:   c = c & A_CHARTEXT;
     }
     return (c);
 }
@@ -341,21 +360,20 @@ VMS_MSG (int status)
 #include <descrip.h>
    char errstr[81], buf[120];
    $DESCRIPTOR(errdesc, errstr);
-   short int length;
+   short length;
 #define err_out(msg) fprintf (stderr,msg)
 
 /* Check for no error or standard error */
 
-   if (~status & 1) {
-      status = status & 0x8000 ? status & 0xFFFFFFF : status & 0xFFFF;
-      if (SYS$GETMSG(status, &length, &errdesc, 1, 0) == SS$_NORMAL) {
-	 errstr[length] = '\0';
-	 (void) sprintf (buf, "<0x%x> %s", status, errdesc.dsc$a_pointer);
-	 err_out (buf);
-      }
-      else
-         err_out ("System error");
-   }
+    if (~status & 1) {
+	status = status & 0x8000 ? status & 0xFFFFFFF : status & 0xFFFF;
+	if (SYS$GETMSG(status, &length, &errdesc, 1, 0) == SS$_NORMAL) {
+	    errstr[length] = '\0';
+	    (void) sprintf(buf, "<0x%x> %s", status, errdesc.dsc$a_pointer);
+	    err_out(buf);
+	} else
+	    err_out("System error");
+    }
 }
 #endif /* VMS */
 
@@ -387,16 +405,16 @@ struct ltchars old_chars, new_chars;
 #endif
 
 char dont_use[] = {
-	ctl('['), ctl('a'), ctl('b'), ctl('c'), ctl('e'), ctl('f'), ctl('g'), ctl('h'),
-	ctl('i'), ctl('j'),  ctl('l'), ctl('m'), ctl('n'), ctl('p'), ctl('q'),
-	ctl('r'), ctl('s'), ctl('t'), ctl('u'), ctl('v'),  ctl('w'), ctl('x'),
-	ctl('z'), 0
+    ctl('['), ctl('a'), ctl('b'), ctl('c'), ctl('e'), ctl('f'), ctl('g'),
+    ctl('h'), ctl('i'), ctl('j'),  ctl('l'), ctl('m'), ctl('n'), ctl('p'),
+    ctl('q'), ctl('r'), ctl('s'), ctl('t'), ctl('u'), ctl('v'),  ctl('w'),
+    ctl('x'), ctl('z'), 0
 };
 
 void
 charout(int c)
 {
-	(void)putchar(c);
+    (void)putchar(c);
 }
 
 void
@@ -499,9 +517,9 @@ nmgetch()
 #endif
 
     if (dumpindex && *dumpindex)
-	    return (*dumpindex++);
+	return (*dumpindex++);
 
-    c = toascii(getchar());
+    c = getchar();
     biggest = 0;
     almost = 0;
 
@@ -513,9 +531,9 @@ nmgetch()
 	    kp->k_index++;
 	    if (kp->k_str[kp->k_index] == 0) {
 		c = kp->k_val;
-	        for (kp = &km[0]; kp < &km[N_KEY]; kp++)
-	            kp->k_index = 0;
-	        return(c);
+		for (kp = &km[0]; kp < &km[N_KEY]; kp++)
+		    kp->k_index = 0;
+		return (c);
 	    }
 	}
 	if (!biggest && kp->k_index)
@@ -531,7 +549,7 @@ nmgetch()
 	if (setjmp(wakeup) == 0) { 
 	    maybe = nmgetch();
 	    (void) alarm(0);
-	    return(maybe);
+	    return (maybe);
 	}
     }
     
@@ -582,66 +600,36 @@ nmgetch()
 
     c = wgetch(stdscr);
     switch (c) {
-#ifdef KEY_LEFT
-    case KEY_LEFT:  c = ctl('b'); break;
-#endif
-#ifdef KEY_RIGHT
-    case KEY_RIGHT: c = ctl('f'); break;
-#endif
-#ifdef KEY_UP
-    case KEY_UP:    c = ctl('p'); break;
-#endif
-#ifdef KEY_DOWN
-    case KEY_DOWN:  c = ctl('n'); break;
-#endif
-#ifdef KEY_BACKSPACE
-    case KEY_BACKSPACE:  c = ctl('h'); break;
-#endif
-#ifdef KEY_NPAGE
-    case KEY_NPAGE:	break;
-#endif
-#ifdef KEY_PPAGE
-    case KEY_PPAGE:	break;
-#endif
-#ifdef KEY_HOME
-    case KEY_HOME:	break;
-#endif
-#ifdef KEY_END
-    case KEY_END:   c = ctl('e'); break;
-#endif
-#ifdef KEY_IC
-    case KEY_IC:    c = 'i'; break;
-#endif
-#ifdef KEY_DC
-    case KEY_DC:    c = 'x'; break;
-#endif
-#ifdef KEY_FIND
-    case KEY_FIND:  c = 'g'; break;
-#endif
-#ifdef KEY_HELP
-    case KEY_HELP:  c = '?'; break;
-#endif
 #ifdef KEY_SELECT
     case KEY_SELECT:c = 'm'; break;
 #endif
 #ifdef KEY_C1
 /* This stuff works for a wyse wy75 in ANSI mode under 5.3.  Good luck. */
 /* It is supposed to map the curses keypad back to the numeric equiv. */
-    case KEY_C1:    c = '0'; break;
-    case KEY_A1:    c = '1'; break;
-    case KEY_B2:    c = '2'; break;
-    case KEY_A3:    c = '3'; break;
-    case KEY_F(5):  c = '4'; break;
-    case KEY_F(6):  c = '5'; break;
-    case KEY_F(7):  c = '6'; break;
-    case KEY_F(9):  c = '7'; break;
-    case KEY_F(10): c = '8'; break;
-    case KEY_F0:    c = '9'; break;
-    case KEY_C3:    c = '.'; break;
-    case KEY_ENTER: c = ctl('m'); break;
+
+/* I had to disable this to make programmable function keys work.  I'm
+ * not familiar with the wyse wy75 terminal.  Does anyone know how to
+ * make this work without causing problems with programmable function
+ * keys on everything else?  - CRM
+
+	case KEY_C1:	c = '0'; break;
+	case KEY_A1:	c = '1'; break;
+	case KEY_B2:	c = '2'; break;
+	case KEY_A3:	c = '3'; break;
+	case KEY_F(5):	c = '4'; break;
+	case KEY_F(6):	c = '5'; break;
+	case KEY_F(7):	c = '6'; break;
+	case KEY_F(9):	c = '7'; break;
+	case KEY_F(10):	c = '8'; break;
+	case KEY_F0:	c = '9'; break;
+	case KEY_C3:	c = '.'; break;
+	case KEY_ENTER:	c = ctl('m'); break;
+
+ *
+ *
+ */
 #endif
-    default:        c = toascii(c); 
-    break;
+	default:	break;
     }
     return (c);
 }
@@ -668,7 +656,7 @@ time_out(int signo)
  */
 
 char *
-strtof(register char *p, double *res)
+strtof(char *p, double *res)
 {
     double acc;
     int sign;
@@ -685,7 +673,7 @@ strtof(register char *p, double *res)
 	error("Floating point exception\n");
 	*res = 0.0; 
         (void) signal(SIGFPE, sig_save);
-	return(p);
+	return (p);
     }
     acc = 0.0;
     sign = 1;
