@@ -15,9 +15,9 @@
  *   -P		Use numbers only when there is no [-+eE] (plain numbers only)
  *
  *  Author: Robert Bond
- *  Adjustments: Jeff Buhrt and Eric Putz
+ *  Adjustments: Jeff Buhrt, Eric Putz and Chuck Martin
  */
-char *rev = "$Revision: 7.2 $";
+char *rev = "$Revision: 7.7 $";
 
 #include <ctype.h>
 #include <stdio.h>
@@ -42,6 +42,12 @@ int *precision;
 int maxcols;
 int *realfmt;
 
+int curlen;
+int curcol, coff;
+int currow, roff;
+int first;
+int effr, effc;
+
 /* option flags reset */
 int colfirst = FALSE;
 int leftadj = FALSE;
@@ -59,20 +65,16 @@ int plainnums	= FALSE;
 
 char token[1000];
 
+int
 main(int argc, char **argv)
 {
-    int curlen;
-    int curcol, coff;
-    int currow, roff;
-    int first;
     int c;
-    register effr, effc;
-    int i,j;
+    int i, j;
     register char *p;
 
     progname = argv[0];
     while ((c = getopt(argc, argv, "rfLks:R:C:n:d:SPv")) != EOF) {
-	switch(c) {
+	switch (c) {
 	case 'r':
 	    colfirst = TRUE;
 	    break;
@@ -130,18 +132,18 @@ main(int argc, char **argv)
     currow = r0; roff = 0;
     first = TRUE;
 
-    while(1) {
+    while (1) {
 
 	effr = currow+roff;
 	effc = curcol+coff;
 
-	switch(scan()) {
+	switch (scan()) {
 	case END:
-	    if(drop_format) exit(0);
+	    if (drop_format) exit(0);
 	    for (i = 0; i<maxcols; i++) {
 		if (fwidth[i])
-		    (void) printf("format %s %d %d\n", coltoa(i), 
-			fwidth[i]+1, precision[i]);
+		    (void) printf("format %s %d %d %d\n", coltoa(i), 
+			fwidth[i]+1, precision[i], REFMTFIX);
 	    }
 	    exit(0);
 	case NUM:
@@ -246,10 +248,10 @@ scan()
     c = getchar();
 
     if (c == EOF)
-	return(END);
+	return (END);
 
     if (c == '\n')
-	return(EOL);
+	return (EOL);
 
     if (c == delim1 || c == delim2) {
         if (strip_delim) {
@@ -257,7 +259,7 @@ scan()
 	        ;
 	    (void)ungetc(c, stdin);
 	} 
-	return(SPACE);
+	return (SPACE);
     }
 
     if (c == '\"') {
@@ -266,7 +268,7 @@ scan()
 	if (c != '\"')
 	    (void)ungetc(c, stdin);
 	*p = '\0';
-	return(ALPHA);
+	return (ALPHA);
     }
 
     while (c != delim1 && c != delim2 && c!= '\n' && c != EOF) {
@@ -287,7 +289,7 @@ scan()
     if (!strnums && (isdigit(c) || c == '.' || c == '-' || c == '+')) {
 	int	lastprtnum = FALSE;
 
-	while(isdigit(c) || c == '.' || (!plainnums && (c == '-' ||
+	while (isdigit(c) || c == '.' || (!plainnums && (c == '-' ||
 					c == '+' || c == 'e' || c == 'E'))) {
 		if (isdigit(c)) 
 			lastprtnum = founddigit = TRUE;
@@ -297,31 +299,31 @@ scan()
 		c = *p++;
 	}
 	if (c == '\0' && founddigit && lastprtnum)
-	    return(NUM);
+	    return (NUM);
 	else
-	    return(ALPHA);
+	    return (ALPHA);
     }
 
-    return(ALPHA);
+    return (ALPHA);
 }
     
 /* turns [A-Z][A-Z] into a number */
 int
 getcol(char *p)
 {
-    register  col;
+    int col;
 
     col = 0;
     if (!p)
-	return(0);
-    while(*p && !isalpha(*p)) 
+	return (0);
+    while (*p && !isalpha(*p)) 
 	p++; 
     if (!*p)
-	return(0);
+	return (0);
     col = ((*p & 0137) - 'A');
     if (isalpha(*++p)) 
 	col = (col + 1)*26 + ((*p & 0137) - 'A');
-    return(col);
+    return (col);
 }
 
 /* given a string turn it into a row number */
@@ -332,16 +334,16 @@ getrow(char *p)
 
     row = 0;
     if (!p)
-	return(0);
-    while(*p && !isdigit(*p))
+	return (0);
+    while (*p && !isdigit(*p))
 	p++; 
     if (!*p)
-	return(0);
-    while(*p && isdigit(*p))
+	return (0);
+    while (*p && isdigit(*p))
     {	row = row * 10 + *p - '0';
 	p++;
     }
-    return(row);
+    return (row);
 }
 
 /* turns a column number into [A-Z][A-Z] */
@@ -360,5 +362,5 @@ coltoa(int col)
     }
     *p++ = col+'A';
     *p = '\0';
-    return(rname);
+    return (rname);
 }
