@@ -1,4 +1,4 @@
- /*	SC	A Spreadsheet Calculator
+/*	SC	A Spreadsheet Calculator
  *		Main driver
  *
  *		original by James Gosling, September 1982
@@ -7,11 +7,12 @@
  *
  *              More mods Robert Bond, 12/86
  *		More mods by Alan Silverstein, 3-4/88, see list of changes.
- *		$Revision: 7.2 $
+ *		$Revision: 7.7 $
  *
  */
 
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <signal.h>
 #include <curses.h>
 #include <ctype.h>
@@ -23,6 +24,14 @@
 #include <string.h>
 #endif
 #endif
+#if defined(BSD42) || defined(BSD43) || defined(VMS)
+#include <sys/file.h>
+#else
+#include <fcntl.h>
+#endif
+#ifndef MSDOS
+#include <unistd.h>
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -43,12 +52,12 @@ extern	void	startdisp(), stopdisp();
 /* Globals defined in sc.h */
 
 struct ent ***tbl;
+int arg = 1;
 int strow = 0, stcol = 0;
 int currow = 0, curcol = 0;
-int savedrow[26], savedcol[26];
-int savedstrow[26], savedstcol[26];
+int savedrow[27], savedcol[27];
+int savedstrow[27], savedstcol[27];
 int FullUpdate = 0;
-int ClearScreen = 0;	/* don't try to be smart */
 int maxrow, maxcol;
 int maxrows, maxcols;
 int *fwidth;
@@ -63,6 +72,15 @@ int modflg;
 int cellassign;
 int numeric;
 char *mdir;
+char *fkey[FKEYS];
+char *scext;
+char *ascext;
+char *tbl0ext;
+char *tblext;
+char *latexext;
+char *slatexext;
+char *texext;
+int scrc = 0;
 int showsc, showsr;	/* Starting cell for highlighted range */
 #ifdef RIGHT_CBUG
 int	wasforw	= FALSE;
@@ -81,11 +99,13 @@ int  showcell  = 1;	/* Causes current cell to be highlighted	  */
 int  showrange = 0;	/* Causes ranges to be highlighted		  */
 int  showneed  = 0;	/* Causes cells needing values to be highlighted  */
 int  showexpr  = 0;	/* Causes cell exprs to be displayed, highlighted */
+int  shownote  = 0;	/* Causes cells with attached notes to be
+			   highlighted					  */
 
-int  autocalc = 1 ;	/* 1 to calculate after each update */
+int  autocalc  = 1;	/* 1 to calculate after each update */
 int  autolabel = 1;     /* If room, causes label to be created after a define*/
 int  calc_order = BYROWS;
-int  optimize = 0;	/* Causes numeric expressions to be optimized */
+int  optimize  = 0;	/* Causes numeric expressions to be optimized */
 int  tbl_style = 0;	/* headers for T command output */
 int  rndtoeven = 0;
 int  numeric_field = 0; /* Started the line editing with a number */
@@ -120,17 +140,18 @@ lookat(int row, int col)
     checkbounds(&row, &col);
     pp = ATBL(tbl, row, col);
     if (*pp == (struct ent *)0) {
-        if (freeents != NULL)
-	{	*pp = freeents;
-		freeents = freeents->next;
-	}
-	else
-		*pp = (struct ent *) scxmalloc((unsigned)sizeof(struct ent));
+        if (freeents != NULL) {
+	    *pp = freeents;
+	    freeents = freeents->next;
+	} else
+	    *pp = (struct ent *) scxmalloc((unsigned)sizeof(struct ent));
 	if (row>maxrow) maxrow = row;
 	if (col>maxcol) maxcol = col;
 	(*pp)->label = (char *)0;
 	(*pp)->row = row;
 	(*pp)->col = col;
+	(*pp)->nrow = -1;
+	(*pp)->ncol = -1;
 	(*pp)->flags = 0;
 	(*pp)->expr = (struct enode *)0;
 	(*pp)->v = (double) 0.0;
@@ -183,7 +204,6 @@ main (int argc, char  **argv)
     int     inloop = 1;
     register int   c;
     int     edistate = -1;
-    int     arg = 1;
     int     narg;
     int     nedistate;
     int	    running;
@@ -200,10 +220,11 @@ main (int argc, char  **argv)
     int Nopt = 0;
     int Copt = 0; 
     int Ropt = 0;
+    int Eopt = 0;
 
     int tempx, tempy; 	/* Temp versions of curx, cury */
 
-#if defined(MSDOS)
+#ifdef MSDOS
     if ((revi = strrchr(argv[0], '\\')) != NULL)
 #else
 #ifdef VMS
@@ -216,7 +237,12 @@ main (int argc, char  **argv)
     else
 	progname = argv[0];
 
+#ifndef MSDOS
+    while (argc > 1 && argv[1][0] == '-' && argv[1][1] != '\0' &&
+	    (argv[1][1] != '-' || argv[1][2] != '\0')) {
+#else 
     while (argc > 1 && argv[1][0] == '-') {
+#endif /* MSDOS */
 	argv++;
 	argc--;
     	switch (argv[0][1]) {
@@ -251,6 +277,7 @@ main (int argc, char  **argv)
 		    break;
 	    case 'e':
 		    rndtoeven = 1;
+		    Eopt = 1;
 		    break;
 	    default:
 		    (void) fprintf(stderr,"%s: unrecognized option: \"%c\"\n",
@@ -259,8 +286,6 @@ main (int argc, char  **argv)
 	}
     }
 
-    *curfile ='\0';
-
     startdisp();
     signals();
 
@@ -271,9 +296,15 @@ main (int argc, char  **argv)
     }
 
     /* unset all marks */
-    for (c = 0; c < 26; c++)
+    for (c = 1; c < 27; c++)
 	savedrow[c] = savedcol[c] = -1;
 
+    /* set location to return to with `` or '' */
+    savedrow[0] = currow;
+    savedcol[0] = curcol;
+    savedstrow[0] = strow;
+    savedstcol[0] = stcol;
+
     /*
      * Build revision message for later use:
      */
@@ -284,11 +315,47 @@ main (int argc, char  **argv)
     revmsg [strlen (revmsg) - 2] = 0;		/* erase last character */
     (void) strcat (revmsg, ":  Type '?' for help.");
 
-    if (argc > 1) {
-	(void) strcpy(curfile,argv[1]);
-	readfile(argv[1], 0);
+#ifndef MSDOS
+    /*
+     * Load $HOME/.scrc if present.
+     */
+    strcpy(curfile, getenv("HOME"));
+    strcat(curfile, "/.scrc");
+    if ((c = open(curfile, O_RDONLY)) > -1) {
+	close(c);
+	readfile(curfile, 0);
     }
 
+    /*
+     * Load ./.scrc if present and $HOME/.scrc contained `set scrc'.
+     */
+    if (scrc && strcmp(getenv("HOME"), getcwd(curfile, PATHLEN)) &&
+	    (c = open(".scrc", O_RDONLY)) > -1) {
+	close(c);
+	readfile(".scrc", 0);
+    }
+#endif /* MSDOS */
+
+    *curfile = '\0';
+
+#ifndef MSDOS
+    if (argc > 1 && !strcmp(argv[1],"--")) {
+	argv++;
+	argc--;
+    }
+    if (argc > 1 && argv[1][0] != '|' &&
+	    (argv[1][0] != '-' || argv[1][1] != '\0'))
+#else 
+    if (argc > 1)
+#endif /* MSDOS */
+	(void) strcpy(curfile,argv[1]);
+    while (argc > 1) {
+	readfile(argv[1], 0);
+	argv++;
+	argc--;
+    }
+    EvalAll();
+
     if (Mopt)
 	autocalc = 0;
     if (Oopt)
@@ -299,11 +366,13 @@ main (int argc, char  **argv)
 	calc_order = BYCOLS;
     if (Ropt)
 	calc_order = BYROWS;
+    if (Eopt)
+	rndtoeven = 1;
 
     modflg = 0;
     cellassign = 0;
 #ifdef VENIX
-    setbuf (stdin, NULL);
+    setbuf(stdin, NULL);
 #endif
     FullUpdate++;
 
@@ -324,8 +393,7 @@ main (int argc, char  **argv)
 
 #ifdef	SIGWINCH
 	/* got a SIGWINCH? */
-	if (hitwinch)
-	{
+	if (hitwinch) {
 	    hitwinch = 0;
 	    stopdisp();
 	    startdisp();
@@ -346,6 +414,7 @@ main (int argc, char  **argv)
 	seenerr = 0;
 	showneed = 0;	/* reset after each update */
 	showexpr = 0;
+	shownote = 0;
 
 	/*
 	 * there seems to be some question about what to do w/ the iscntrl
@@ -372,7 +441,6 @@ main (int argc, char  **argv)
 		    showneed = 1;
 		case ctl('l'):
 		    FullUpdate++;
-		    ClearScreen++;
 		    (void) clearok(stdscr,1);
 		    break;
 		case ctl('x'):
@@ -391,21 +459,27 @@ main (int argc, char  **argv)
 			numeric_field = 0;
 		    }
 		    craction = c;
-		    backcol(arg);
+		    if (linelim < 0) {
+			backcol(arg);
+			break;
+		    }
+		    write_line(ctl('b'));
 		    break;
 		case ctl('c'):
 		    running = 0;
 		    break;
 
+#ifdef KEY_END
+		case KEY_END:
+#endif
 		case ctl('e'):
-
 		    switch (nmgetch()) {
-		    case ctl('p'): case 'k':	doend (-1, 0);	break;
-		    case ctl('n'): case 'j':	doend ( 1, 0);	break;
+		    case ctl('p'): case 'k':	doend(-1, 0);	break;
+		    case ctl('n'): case 'j':	doend( 1, 0);	break;
 		    case ctl('b'): case 'h':
-		    case ctl('h'):		doend ( 0,-1);	break;
+		    case ctl('h'):		doend( 0,-1);	break;
 		    case ctl('f'): case 'l':
-		    case ctl('i'): case ' ':	doend ( 0, 1);	break;
+		    case ctl('i'): case ' ':	doend( 0, 1);	break;
 
 		    case ESC:
 		    case ctl('g'):
@@ -426,17 +500,21 @@ main (int argc, char  **argv)
 			numeric_field = 0;
 		    }
 		    craction = c;
-		    forwcol(arg);
+		    if (linelim < 0) {
+			forwcol(arg);
 #ifdef RIGHT_CBUG
-		    wasforw++;
+			wasforw++;
 #endif
+			break;
+		    }
+		    write_line(ctl('f'));
 		    break;
 
 		case ctl('g'):
 		    showrange = 0;
 		    linelim = -1;
-		    (void) move (1, 0);
-		    (void) clrtoeol ();
+		    (void) move(1, 0);
+		    (void) clrtoeol();
 		    break;
 
 		case ESC:	/* ctl('[') */
@@ -447,6 +525,9 @@ main (int argc, char  **argv)
 		    write_line(ctl('d'));
 		    break;
 
+#ifdef KEY_BACKSPACE
+		case KEY_BACKSPACE:
+#endif
 		case DEL:
 		case ctl('h'):
 		    if (linelim < 0) {	/* not editing line */
@@ -461,16 +542,7 @@ main (int argc, char  **argv)
 			forwcol(arg);
 			break;
 		    }
-		    if (!showrange) {
-			startshow();
-		    } else {
-			showdr();
-			linelim = strlen(line);
-			line[linelim++] = ' ';
-			line[linelim] = '\0';
-			showrange = 0;
-		    }
-		    linelim = strlen(line);
+		    write_line(ctl('i'));
 		    break;
 
 		case ctl('m'):
@@ -487,7 +559,11 @@ main (int argc, char  **argv)
 			numeric_field = 0;
 		    }
 		    craction = c;
-		    forwrow(arg);
+		    if (linelim < 0) {
+			forwrow(arg);
+			break;
+		    }
+		    write_line(ctl('n'));
 		    break;
 
 		case ctl('p'):
@@ -498,7 +574,11 @@ main (int argc, char  **argv)
 			numeric_field = 0;
 		    }
 		    craction = c;
-		    backrow(arg);
+		    if (linelim < 0) {
+			backrow(arg);
+			break;
+		    }
+		    write_line(ctl('p'));
 		    break;
 
 		case ctl('q'):
@@ -619,23 +699,24 @@ main (int argc, char  **argv)
 		    nedistate = 1;
 		    break;
 
-		case ctl('v'):	/* insert variable name */
-		    if (linelim > 0)
-		        ins_string(v_name(currow, curcol));
+		case ctl('v'):	/* switch to navigate mode, or if already *
+				 * in navigate mode, insert variable name */
+		    if (linelim >= 0)
+		        write_line(ctl('v'));
 		    break;
 
 		case ctl('w'):	/* insert variable expression */
-		    if (linelim > 0)  {
+		    if (linelim >= 0)  {
 			static	char *temp = NULL, *temp1 = NULL;
 			static	unsigned	templen = 0;
 			int templim;
 
 			/* scxrealloc will scxmalloc if needed */
-			if (strlen(line)+1 > templen)
-			{	templen = strlen(line)+40;
+			if (strlen(line)+1 > templen) {
+			    templen = strlen(line)+40;
 
-				temp = scxrealloc(temp, templen);
-				temp1= scxrealloc(temp1, templen);
+			    temp = scxrealloc(temp, templen);
+			    temp1= scxrealloc(temp1, templen);
 			}
 			strcpy(temp, line);
 			templim = linelim;
@@ -649,21 +730,21 @@ main (int argc, char  **argv)
 		    break;
 
 		case ctl('a'):	/* insert variable value */
-		    if (linelim > 0) {
+		    if (linelim >= 0) {
 			struct ent *p = *ATBL(tbl, currow, curcol);
 			char temp[100];
 
-			if (p && p -> flags & is_valid) {
+			if (p && p->flags & is_valid) {
 			    (void) sprintf(temp, "%.*f",
-					precision[curcol],p -> v);
+				    precision[curcol], p->v);
 			    ins_string(temp);
 			}
 		    }
 		    break;
 
 	    } /* End of the control char switch stmt */
-	else if (isascii(c) && isdigit(c) && ((numeric && edistate >= 0) ||
-			(!numeric && (linelim < 0 || edistate >= 0)))) {
+	else if (isascii(c) && isdigit(c) && (((!numeric && (linelim < 0)) ||
+		edistate >= 0))) {
 	    /* we got a leading number */
 	    if (edistate != 0) {
 		/* First char of the count */
@@ -680,18 +761,19 @@ main (int argc, char  **argv)
 		nedistate = 0;
 		narg = arg * 10 + (c - '0');
 	    }
+	} else if (c == KEY_F(1) && !fkey[c - KEY_F0 - 1]) {
+	    deraw();
+	    system("man sc");
+	    goraw();
 	} else if (linelim >= 0) {
 	    /* Editing line */
-	    switch(c) {
-	    case ')':
-		if (showrange) {
-		    showdr();
-		    showrange = 0;
-		    linelim = strlen (line);
-		}
-		break;
-	    default:
-		break;
+	    switch (c) {
+		case ')':
+		    if (showrange)
+			showdr();
+		    break;
+		default:
+		    break;
 	    }
 	    write_line(c);
 
@@ -707,12 +789,34 @@ main (int argc, char  **argv)
 	    FullUpdate++;
 	    modflg++;
 	    if (c == '+')
-	    	p -> v += (double) arg;
+	    	p->v += (double) arg;
 	    else
-		p -> v -= (double) arg;
+		p->v -= (double) arg;
+	} else if (c >= KEY_F0 && c <= KEY_F(FKEYS)) {
+	    /* a function key was pressed */
+	    if (fkey[c - KEY_F0 - 1]) {
+		char *tpp;
+
+		write_line(ctl('m'));	/* switch to insert mode */
+		strcpy(line, fkey[c - KEY_F0 - 1]);
+		for (tpp = line; *tpp != '\0'; tpp++)
+		    if (*tpp == '\\' && *(tpp + 1) == '"')
+			memmove(tpp, tpp + 1, strlen(tpp));
+		for (tpp = line; *tpp != '\0'; tpp++) {
+		    char mycell[9];
+		    strcpy(mycell, coltoa(curcol));
+		    sprintf(mycell + strlen(mycell), "%d", currow);
+		    if (*tpp == '$' && *(tpp + 1) == '$') {
+			memmove(tpp + strlen(mycell), tpp + 2, strlen(tpp + 1));
+			memcpy(tpp, mycell, strlen(mycell));
+			tpp += strlen(mycell);
+		    }
+		}
+		write_line(ctl('m'));
+	    }
 	} else
 	    /* switch on a normal command character */
-	    switch(c) {
+	    switch (c) {
 		case ':':
 		    break;	/* Be nice to vi users */
 
@@ -729,7 +833,7 @@ main (int argc, char  **argv)
 			break;
 		    numeric_field = 1;
 		    (void) sprintf(line,"let %s = %c",
-				v_name(currow, curcol), c);
+			    v_name(currow, curcol), c);
 		    linelim = strlen(line);
 		    insert_mode();
 		    break;
@@ -737,8 +841,7 @@ main (int argc, char  **argv)
 		case '=':
 		    if (locked_cell(currow, curcol))
 			break;
-		    (void) sprintf(line,"let %s = ",
-					v_name(currow, curcol));
+		    (void) sprintf(line,"let %s = ", v_name(currow, curcol));
 		    linelim = strlen(line);
 		    insert_mode();
 		    break;
@@ -766,32 +869,31 @@ main (int argc, char  **argv)
 		    (void) fflush(stdout);
 		    (void) fgets(cmd, MAXCMD, stdin);
 		    cmd[strlen(cmd) - 1] = '\0';	/* clobber \n */
-		    if(strcmp(cmd,"!") == 0)		/* repeat? */
-			    (void) strcpy(cmd, lastcmd);
+		    if (strcmp(cmd,"!") == 0)		/* repeat? */
+			(void) strcpy(cmd, lastcmd);
 		    else
-			    (void) strcpy(lastcmd, cmd);
+			(void) strcpy(lastcmd, cmd);
 
-		    if (modflg)
-		    {
+		    if (modflg) {
 			(void) puts ("[No write since last change]");
 			(void) fflush(stdout);
 		    }
 
-		    if (!(pid = fork()))
-		    {
+		    if (!(pid = fork())) {
 			(void) signal(SIGINT, SIG_DFL);  /* reset */
 			if(strlen(cmd))
-				(void) execl(shl,shl,"-c",cmd,(char *)0);
+			    (void) execl(shl,shl,"-c",cmd,(char *)0);
 			else
-				(void) execl(shl, shl, (char *)0);
+			    (void) execl(shl, shl, (char *)0);
 			exit(-127);
 		    }
 
 		    while (pid != wait(&temp));
 
-		    (void) printf("Press RETURN to continue ");
+		    (void) printf("Press any key to continue ");
 		    fflush(stdout);
-		    (void)nmgetch();
+		    cbreak();
+		    (void)getch();
 		    goraw();
 #endif /* VMS */
 		    break;
@@ -802,6 +904,7 @@ main (int argc, char  **argv)
 		 */
 
 		case '/':
+		    insert_mode();
 		    error(
 "Range: x:erase v:value c:copy f:fill d:def l:lock U:unlock s:show u:undef F:fmt");
 		    (void) refresh();
@@ -811,50 +914,68 @@ main (int argc, char  **argv)
 			(void) sprintf(line,"lock [range] ");
 			linelim = strlen(line);
 			startshow();
-			insert_mode();
 			break;
 		    case 'U':
 			(void) sprintf(line,"unlock [range] ");
 			linelim = strlen(line);
 			startshow();
-			insert_mode();
 			break;
 		    case 'c':
 			(void) sprintf(line,"copy [dest_range src_range] ");
 			linelim = strlen(line);
 			startshow();
-			insert_mode();
 			break;
 		    case 'x':
 			(void) sprintf(line,"erase [range] ");
 			linelim = strlen(line);
 			startshow();
-			insert_mode();
 			break;
 		    case 'v':
 			(void) sprintf(line, "value [range] ");
 			linelim = strlen(line);
 			startshow();
-			insert_mode();
 			break;
 		    case 'f':
 			(void) sprintf(line,"fill [range start inc] ");
 			linelim = strlen(line);
 			startshow();
-			insert_mode();
 			break;
 		    case 'd':
 			(void) sprintf(line,"define [string range] \"");
 			linelim = strlen(line);
-			startshow();
 			insert_mode();
-			modflg++;
 			break;
 		    case 'u':
 			(void) sprintf(line,"undefine [range] ");
 			linelim = strlen(line);
+			startshow();
+			break;
+		    case 'r':
+			error("frame (top/bottom/left/right/all/unframe)");
+			refresh();
+			switch (nmgetch()) {
+			case 't':
+			    sprintf(line,"frametop [<outrange> #rows] ");
+			    break;
+			case 'b':
+			    sprintf(line,"framebottom [<outrange> #rows] ");
+			    break;
+			case 'l':
+			    sprintf(line,"frameleft [<outrange> #cols] ");
+			    break;
+			case 'r':
+			    sprintf(line,"frameright [<outrange> #cols] ");
+			    break;
+			case 'a':
+			    sprintf(line,"frame [<outrange> inrange] ");
+			    break;
+			case 'u':
+			    sprintf(line,"unframe [<range>] ");
+			    break;
+			}
+			error("");
+			linelim = strlen(line);
 			insert_mode();
-			modflg++;
 			break;
 		    case 's':
 			if (are_ranges()) {
@@ -863,17 +984,17 @@ main (int argc, char  **argv)
 			    char px[MAXCMD];
 			    char *pager;
 
-			    (void) strcpy(px, "| sort | ");
+			    (void) strcpy(px, "| ");
 			    if (!(pager = getenv("PAGER")))
 				pager = DFLT_PAGER;
-			    (void) strcat(px,pager);
-			    f = openout(px, &pid);
+			    (void) strcat(px, pager);
+			    f = openfile(px, &pid, NULL);
 			    if (!f) {
-				error("Can't open pipe to sort");
+				error("Can't open pipe to %s", pager);
 				break;
 			    }
 			    list_range(f);
-			    closeout(f, pid);
+			    closefile(f, pid, 0);
 			}
 			else error("No ranges defined");
 			break;
@@ -881,10 +1002,10 @@ main (int argc, char  **argv)
 			(void) sprintf(line, "fmt [range \"format\"] ");
 			linelim = strlen(line);
 			startshow();
-			insert_mode();
 			break;
 		    case ESC:
 		    case ctl('g'):
+			error("");
 			break;
 		    default:
 			error("Invalid region command");
@@ -896,6 +1017,9 @@ main (int argc, char  **argv)
 		 * Row/column commands:
 		 */
 
+#ifdef KEY_IC
+		case KEY_IC:
+#endif
 		case 'i':
 		case 'a':
 		case 'd':
@@ -904,64 +1028,64 @@ main (int argc, char  **argv)
 		case 'z':
 		case 's':
 		    {
-			register rcqual;
+			int rcqual;
 
-			if (! (rcqual = get_rcqual(c))) {
+			if (!(rcqual = get_rcqual(c))) {
 			    error ("Invalid row/column command");
 			    break;
 			}
 
-			error ("");	/* clear line */
+			error("");	/* clear line */
 
 			if (rcqual == ESC || rcqual == ctl('g'))
 			    break;
 
 			switch (c) {
 
-			case 'i':
-			    if (rcqual == 'r')	insertrow(arg);
-			    else		opencol(curcol, arg);
-			    break;
+			    case 'i':
+				if (rcqual == 'r')	insertrow(arg);
+				else			insertcol(arg);
+				break;
 
-			case 'a':
-			    if (rcqual == 'r')	while (arg--) duprow();
-			    else		while (arg--) dupcol();
-			    break;
+			    case 'a':
+				if (rcqual == 'r')	while (arg--) duprow();
+				else			while (arg--) dupcol();
+				break;
 
-			case 'd':
-			    if (rcqual == 'r')	deleterow(arg);
-			    else		closecol(curcol, arg);
-			    break;
+			    case 'd':
+				if (rcqual == 'r')	deleterow(arg);
+				else			closecol(curcol, arg);
+				break;
 
-			case 'p':
-			    while (arg--)	pullcells(rcqual);
-			    break;
+			    case 'p':
+				while (arg--)		pullcells(rcqual);
+				break;
 
-			/*
-			 * turn an area starting at currow/curcol into
-			 * constants vs expressions - not reversable
-			 */
-			case 'v':
-			    if (rcqual == 'r')
-				valueize_area(currow, 0,
+			    /*
+			     * turn an area starting at currow/curcol into
+			     * constants vs expressions - not reversable
+			     */
+			    case 'v':
+				if (rcqual == 'r')
+				    valueize_area(currow, 0,
 					      currow + arg - 1, maxcol);
-			    else
-				valueize_area(0, curcol,
+				else
+				    valueize_area(0, curcol,
 					      maxrow, curcol + arg - 1);
-			    modflg = 1;
-			    break;
+				modflg = 1;
+				break;
 
-			case 'z':
-			    if (rcqual == 'r')	hiderow(arg);
-			    else		hidecol(arg);
-			    break;
+			    case 'z':
+				if (rcqual == 'r')	hiderow(arg);
+				else			hidecol(arg);
+				break;
 
-			case 's':
+			    case 's':
 			    /* special case; no repeat count */
 
-			    if (rcqual == 'r')	rowshow_op();
-			    else		colshow_op();
-			    break;
+				if (rcqual == 'r')	rowshow_op();
+				else			colshow_op();
+				break;
 			}
 			break;
 		    }
@@ -1028,7 +1152,7 @@ main (int argc, char  **argv)
 					;
 				    curcol = maxcols - 1;
 				} else {
-				    error ("At start of table");
+				    error("At start of table");
 				    break;
 				}
 			    }
@@ -1044,24 +1168,27 @@ main (int argc, char  **argv)
 		    rowsinrange = 1;
 		    colsinrange = fwidth[curcol];
 		    break;
+#ifdef KEY_HELP
+		case KEY_HELP:
+#endif
 		case '?':
 		    help();
 		    break;
 		case '"':
 		    if (!locked_cell(currow, curcol)) {
-		       (void) sprintf(line, "label %s = \"",
+			(void) sprintf(line, "label %s = \"",
 				v_name(currow, curcol));
-		       linelim = strlen(line);
-		       insert_mode();
+			linelim = strlen(line);
+			insert_mode();
 		    }
 		    break;
 
 		case '<':
 		    if (!locked_cell(currow, curcol)) {
-		       (void) sprintf(line, "leftstring %s = \"",
-			       v_name(currow, curcol));
-		       linelim = strlen(line);
-		       insert_mode();
+			(void) sprintf(line, "leftstring %s = \"",
+				v_name(currow, curcol));
+			linelim = strlen(line);
+			insert_mode();
 		    }
 		    break;
 
@@ -1106,56 +1233,118 @@ main (int argc, char  **argv)
 			(void) sprintf(line, "fmt [format] %s \"%s",
 			    v_name(currow, curcol), p->format);
 			edit_mode();
+			linelim = strlen(line) - 1;
 		    } else {
 			(void) sprintf(line, "fmt [format] %s \"",
 				   v_name(currow, curcol));
 			insert_mode();
+			linelim = strlen(line);
 		    }
-		    linelim = strlen(line);
 		    break;
 		}
+#ifdef KEY_FIND
+		case KEY_FIND:
+#endif
 		case 'g':
-		    (void) sprintf (line, "goto [v] ");
-		    linelim = strlen (line);
+		    (void) sprintf(line, "goto [v] ");
+		    linelim = strlen(line);
 		    insert_mode();
 		    break;
 		case 'P':
-		    (void) sprintf (line, "put [\"dest\" range] \"");
+		    (void) sprintf(line, "put [\"dest\" range] \"");
+
+/* See the comments under "case 'W':" below for an explanation of the
+ * logic here.
+ */
+		    curfile[strlen(curfile) + 1] = '\0';
+		    if (strrchr(curfile, '.') != NULL) {
+			if (!strcmp((strrchr(curfile, '.')), ".sc")) {
+			    *strrchr(curfile, '.') = '\0';
+			    strcpy(curfile + strlen(curfile) + 3, ".\0");
+			} else if (scext != NULL &&
+				!strcmp((strrchr(curfile, '.') + 1), scext)) {
+			    *strrchr(curfile, '.') = '\0';
+			    strcpy(curfile + strlen(curfile) +
+				    strlen(scext) + 1, ".\0");
+			}
+		    }
 		    if (*curfile)
-			error ("Default path is \"%s\"",curfile);
-		    linelim = strlen (line);
+			error("Default path is \"%s.%s\"", curfile,
+				scext == NULL ? "sc" : scext);
+		    c = *(curfile + strlen(curfile) +
+			    strlen(curfile + strlen(curfile) + 1));
+		    *(curfile + strlen(curfile) +
+			    strlen(curfile + strlen(curfile) + 1)) = '\0';
+		    curfile[strlen(curfile)] = c;
+		    linelim = strlen(line);
 		    insert_mode();
 		    break;
 		case 'M':
-		    (void) sprintf (line, "merge [\"source\"] \"");
-		    linelim = strlen (line);
+		    (void) sprintf(line, "merge [\"source\"] \"");
+		    linelim = strlen(line);
 		    insert_mode();
 		    break;
 		case 'R':
 		    if (mdir)
-			(void) sprintf (line,"merge [\"macro_file\"] \"%s/", mdir);
+			(void) sprintf(line,"merge [\"macro_file\"] \"%s", mdir);
 		    else
-			(void) sprintf (line,"merge [\"macro_file\"] \"");
-		    linelim = strlen (line);
+			(void) sprintf(line,"merge [\"macro_file\"] \"");
+		    linelim = strlen(line);
 		    insert_mode();
 		    break;
 		case 'D':
-		    (void) sprintf (line, "mdir [\"macro_directory\"] \"");
-		    linelim = strlen (line);
+		    (void) sprintf(line, "mdir [\"macro_directory\"] \"");
+		    linelim = strlen(line);
 		    insert_mode();
 		    break;
 		case 'G':
-		    (void) sprintf (line, "get [\"source\"] \"");
+		    (void) sprintf(line, "get [\"source\"] \"");
 		    if (*curfile)
-			error ("Default file is \"%s\"",curfile);
+			error("Default file is \"%s\"", curfile);
 		    linelim = strlen (line);
 		    insert_mode();
 		    break;
 		case 'W':
-		    (void) sprintf (line, "write [\"dest\" range] \"");
+		    (void) sprintf(line, "write [\"dest\" range] \"");
+
+/* First, append an extra null byte to curfile.  Then, if curfile ends in
+ * ".sc" (or '.' followed by the string in scext), move the '.' to the
+ * end and replace it with a null byte.  This results in two consecutive
+ * null-terminated strings, the first being curfile with the ".sc" (or '.'
+ * and scext) removed, if present, and the second being either "sc." (or
+ * scext and '.') or "", depending on whether the ".sc" (or '.' and scext)
+ * was present or not.
+ */
+		    curfile[strlen(curfile) + 1] = '\0';
+		    if (strrchr(curfile, '.') != NULL) {
+			if (!strcmp((strrchr(curfile, '.')), ".sc")) {
+			    *strrchr(curfile, '.') = '\0';
+			    strcpy(curfile + strlen(curfile) + 3, ".\0");
+			} else if (scext != NULL &&
+				!strcmp((strrchr(curfile, '.') + 1), scext)) {
+			    *strrchr(curfile, '.') = '\0';
+			    strcpy(curfile + strlen(curfile) +
+				    strlen(scext) + 1, ".\0");
+			}
+		    }
+
+/* Now append ".asc" (or '.' and the value of ascext) to the possibly
+ * truncated curfile.
+ */
 		    if (*curfile)
-                       error ("Default file is \"%s.asc\"",curfile);
-		    linelim = strlen (line);
+			error("Default file is \"%s.%s\"", curfile,
+				ascext == NULL ? "asc" : ascext);
+
+/* Now swap the '.' and null bytes again.  If there is no '.', swap a
+ * null byte with itself.  This may seem convoluted, but it works well,
+ * and obviates the need for a 1024 byte temporary buffer. - CRM
+ */
+		    c = *(curfile + strlen(curfile) +
+			    strlen(curfile + strlen(curfile) + 1));
+		    *(curfile + strlen(curfile) +
+			    strlen(curfile + strlen(curfile) + 1)) = '\0';
+		    curfile[strlen(curfile)] = c;
+		    linelim = strlen(line);
 		    insert_mode();
 		    break;
 		case 'S':	/* set options */
@@ -1166,19 +1355,48 @@ main (int argc, char  **argv)
 		    break;
 		case 'T':	/* tbl output */
 		    (void) sprintf (line, "tbl [\"dest\" range] \"");
+
+/* See the comments under "case 'W':" above for an explanation of the
+ * logic here.
+ */
+		    curfile[strlen(curfile) + 1] = '\0';
+		    if (strrchr(curfile, '.') != NULL) {
+			if (!strcmp((strrchr(curfile, '.')), ".sc")) {
+			    *strrchr(curfile, '.') = '\0';
+			    strcpy(curfile + strlen(curfile) + 3, ".\0");
+			} else if (scext != NULL &&
+				!strcmp((strrchr(curfile, '.') + 1), scext)) {
+			    *strrchr(curfile, '.') = '\0';
+			    strcpy(curfile + strlen(curfile) +
+				    strlen(scext) + 1, ".\0");
+			}
+		    }
 		    if (*curfile && tbl_style == 0)
-                       error ("Default file is \"%s.cln\"",curfile);
-                    else if (*curfile && tbl_style == TBL)
-                       error ("Default file is \"%s.tbl\"",curfile);
-                    else if (*curfile && tbl_style == LATEX)
-                       error ("Default file is \"%s.lat\"",curfile);
-                    else if (*curfile && tbl_style == SLATEX)
-                       error ("Default file is \"%s.stx\"",curfile);
-                    else if (*curfile && tbl_style == TEX)
-                       error ("Default file is \"%s.tex\"",curfile);
+			error("Default file is \"%s.%s\"", curfile,
+				tbl0ext == NULL ? "cln" : tbl0ext);
+		    else if (*curfile && tbl_style == TBL)
+			error("Default file is \"%s.%s\"", curfile,
+				tblext == NULL ? "tbl" : tblext);
+		    else if (*curfile && tbl_style == LATEX)
+			error("Default file is \"%s.%s\"", curfile,
+				latexext == NULL ? "lat" : latexext);
+		    else if (*curfile && tbl_style == SLATEX)
+			error("Default file is \"%s.%s\"", curfile,
+				slatexext == NULL ? "stx" : slatexext);
+		    else if (*curfile && tbl_style == TEX)
+			error("Default file is \"%s.%s\"", curfile,
+				texext == NULL ? "tex" : texext);
+		    c = *(curfile + strlen(curfile) +
+			    strlen(curfile + strlen(curfile) + 1));
+		    *(curfile + strlen(curfile) +
+			    strlen(curfile + strlen(curfile) + 1)) = '\0';
+		    curfile[strlen(curfile)] = c;
 		    linelim = strlen (line);
 		    insert_mode();
 		    break;
+#ifdef KEY_DC
+		case KEY_DC:
+#endif
 		case 'x':
 		    {
 		    register struct ent **pp;
@@ -1186,26 +1404,25 @@ main (int argc, char  **argv)
 
 		    flush_saved();
 		    if(calc_order == BYROWS) {
-		      for (c1 = curcol; arg-- && c1 < maxcols; c1++) {
-			pp = ATBL(tbl, currow, c1);
-		        if ((*pp) && !locked_cell(currow, curcol)) {
-			   if (*pp) {
-			       free_ent(*pp);
-			       *pp = (struct ent *)0;
-			   }
-		        }
-		      }
-		    }
-		    else {
-		      for (c1 = currow; arg-- && c1 < maxrows; c1++) {
-			pp = ATBL(tbl, c1, curcol);
-		        if ((*pp) && !locked_cell(currow, curcol)) {
-			   if (*pp) {
-			       free_ent(*pp);
-			       *pp = (struct ent *)0;
-			   }
-		        }
-		      }
+			for (c1 = curcol; arg-- && c1 < maxcols; c1++) {
+			    pp = ATBL(tbl, currow, c1);
+			    if ((*pp) && !locked_cell(currow, curcol)) {
+				if (*pp) {
+				    free_ent(*pp);
+				    *pp = (struct ent *)0;
+				}
+			    }
+			}
+		    } else {
+			for (c1 = currow; arg-- && c1 < maxrows; c1++) {
+			    pp = ATBL(tbl, c1, curcol);
+			    if ((*pp) && !locked_cell(currow, curcol)) {
+				if (*pp) {
+				    free_ent(*pp);
+				    *pp = (struct ent *)0;
+				}
+			    }
+			}
 		    }
 		    sync_refs();
 		    modflg++;
@@ -1216,12 +1433,21 @@ main (int argc, char  **argv)
 		case 'q':
 		    running = 0;
 		    break;
+#ifdef KEY_LEFT
+		case KEY_LEFT:
+#endif
 		case 'h':
 		    backcol(arg);
 		    break;
+#ifdef KEY_DOWN
+		case KEY_DOWN:
+#endif
 		case 'j':
 		    forwrow(arg);
 		    break;
+#ifdef KEY_UP
+		case KEY_UP:
+#endif
 		case 'k':
 		    backrow(arg);
 		    break;
@@ -1259,17 +1485,20 @@ main (int argc, char  **argv)
 		case 'L':
 			forwcol(lcols -(curcol-stcol)+1);
 			break;
+#ifdef KEY_RIGHT
+		case KEY_RIGHT:
+#endif
 		case ' ':
 		case 'l':
 		    forwcol(arg);
 		    break;
 		case 'm':
-		    error("Mark cell (select a-z)");
+		    error("Mark cell:");
 		    if ((c=nmgetch()) == ESC || c == ctl('g')) {
 			error("");
 			break;
 		    }
-		    if ((c -= 'a') < 0 || c > 25) {
+		    if ((c -= ('a' - 1)) < 1 || c > 26) {
 			error("Invalid character for mark (must be a-z)");
 			break;
 		    }
@@ -1280,12 +1509,12 @@ main (int argc, char  **argv)
 		    savedstcol[c] = stcol;
 		    break;
 		case 'c':
-		    error("Copy marked cell (select a-z)");
+		    error("Copy marked cell:");
 		    if ((c=nmgetch()) == ESC || c == ctl('g')) {
 			error("");
 			break;
 		    }
-		    if ((c -= 'a') < 0 || c > 25) {
+		    if ((c -= ('a' - 1)) < 1 || c > 26) {
 			error("Invalid character for mark (must be a-z)");
 			break;
 		    }
@@ -1296,14 +1525,14 @@ main (int argc, char  **argv)
 		    error("");
 		    {
 			register struct ent *p = *ATBL(tbl, savedrow[c], savedcol[c]);
-			register c1;
+			int c1;
 			register struct ent *n;
 			if (!p)
 			    break;
 			FullUpdate++;
 			modflg++;
 			for (c1 = curcol; arg-- && c1 < maxcols; c1++) {
-			    n = lookat (currow, c1);
+			    n = lookat(currow, c1);
 			    (void) clearent(n);
 			    copyent(n, p, currow - savedrow[c], c1 - savedcol[c]);
 			}
@@ -1312,14 +1541,25 @@ main (int argc, char  **argv)
 		case '`': case '\'':
 		    {
 			char c1 = c;
-			error("Go to marked cell (select a-z)");
+
+			/* Remember the current position */
+			int tmprow = currow;
+			int tmpcol = curcol;
+			int tmpstrow = strow;
+			int tmpstcol = stcol;
+
+			error("Go to marked cell:");
 			if ((c = nmgetch()) == ESC || c == ctl('g')) {
 			    error("");
 			    break;
 			}
-			if ((c -= 'a') < 0 || c > 25) {
-			    error("Invalid character for mark (must be a-z)");
-			    break;
+			if ((c -= ('a' - 1)) < 1 || c > 26) {
+			    if (c1 == '`' || c1 == '\'')
+				c = 0;
+			    else {
+				error("Invalid character for mark (must be a-z, ` or \')");
+				break;
+			    }
 			}
 			if (savedrow[c] == -1) {
 			    error("Mark not set");
@@ -1334,9 +1574,59 @@ main (int argc, char  **argv)
 			    strow = savedstrow[c];
 			    stcol = savedstcol[c];
 			}
+			savedrow[0] = tmprow;
+			savedcol[0] = tmpcol;
+			savedstrow[0] = tmpstrow;
+			savedstcol[0] = tmpstcol;
+
 			FullUpdate++;
 			break;
 		    }
+		case 'n':
+		    {
+			register struct ent *p;
+
+			error("Note: Add/Delete/Show?");
+			if ((c = nmgetch()) == ESC || c == ctl('g')) {
+			    error("");
+			    break;
+			}
+			if (c == 'a' || c == 'A') {
+			    sprintf(line, "addnote [target range] %s ", 
+				    v_name(currow, curcol));
+			    linelim = strlen(line);
+			    insert_mode();
+			    error("");
+			    break;
+			}
+			if (c == 'd' || c == 'D') {
+			    p = lookat(currow, curcol);
+			    p->nrow = p->ncol = -1;
+			    error("");
+			    break;
+			}
+			if (c == 's' || c == 'S') {
+			    FullUpdate++;
+			    shownote = 1;
+			    clearok(stdscr,1);
+			    error("Highlighted cells have attached notes.");
+			    break;
+			}
+			error("Invalid command");
+			break;
+		    }
+		case '*':
+		    {
+			register struct ent *p;
+
+			p = lookat(currow, curcol);
+			if (p->nrow == -1)
+			    error("No note attached");
+			else
+			    moveto(p->nrow, p->ncol, p->nlastrow, p->nlastcol,
+				    -1, -1);
+			break;
+		    }
 		case 'C':
 		    /* Force centering of current cell (or range, if we've
 		     * just jumped to a new range with the goto command).
@@ -1344,7 +1634,6 @@ main (int argc, char  **argv)
 		    strow = -1;
 		    stcol = -1;
 		    FullUpdate++;
-		    ClearScreen++;
 		    (void) clearok(stdscr,1);
 		    break;
 		default:
@@ -1369,33 +1658,11 @@ main (int argc, char  **argv)
     /*NOTREACHED*/
 }
 
-/* show the current range (see ^I), we are moving around to define a range */
-void
-startshow()
-{
-    showrange = 1;
-    showsr = currow;
-    showsc = curcol;
-}
-
-/* insert the range we defined by moving around the screen, see startshow() */
-void
-showdr()
-{
-    int     minsr, minsc, maxsr, maxsc;
-
-    minsr = showsr < currow ? showsr : currow;
-    minsc = showsc < curcol ? showsc : curcol;
-    maxsr = showsr > currow ? showsr : currow;
-    maxsc = showsc > curcol ? showsc : curcol;
-    (void) sprintf (line+linelim,"%s", r_name(minsr, minsc, maxsr, maxsc));
-}
-
 /* set the calculation order */
 void
 setorder(int i)
 {
-	if((i == BYROWS)||(i == BYCOLS))
+	if ((i == BYROWS) || (i == BYCOLS))
 	    calc_order = i;
 }
 
@@ -1522,26 +1789,26 @@ modcheck(char *endstr)
 int
 locked_cell(int r, int c)
 {
-	struct ent *p = *ATBL(tbl, r, c);
-	if (p && (p->flags & is_locked)) {
-		error("Cell %s%d is locked", coltoa(c), r) ;
-		return(1);
-	}
-	return(0);
+    struct ent *p = *ATBL(tbl, r, c);
+    if (p && (p->flags & is_locked)) {
+	error("Cell %s%d is locked", coltoa(c), r) ;
+	return(1);
+    }
+    return(0);
 }
 
 /* Check if area contains locked cells */
 int
 any_locked_cells(int r1, int c1, int r2, int c2)
 {
-	int r, c;
-	struct ent *p ;
+    int r, c;
+    struct ent *p ;
 
-	for (r=r1; r<=r2; r++)
-		for (c=c1; c<=c2; c++) {
-			p = *ATBL(tbl, r, c);
-			if (p && (p->flags&is_locked))
-				return(1);
-		}
-	return(0);
+    for (r=r1; r<=r2; r++)
+	for (c=c1; c<=c2; c++) {
+	    p = *ATBL(tbl, r, c);
+	    if (p && (p->flags&is_locked))
+		return(1);
+	}
+    return(0);
 }
