@@ -6,11 +6,11 @@
  *			University of Maryland
  *		R. Bond  12/86
  *		More mods by Alan Silverstein, 3-4/88, see list of changes.
- *		$Revision: 7.2 $
+ *		$Revision: 7.7 $
  *
  */
 
-#if defined(MSDOS)
+#ifdef MSDOS
 #include <stdio.h>
 #endif
 
@@ -22,7 +22,6 @@
 
 #define CRROWS 1
 #define CRCOLS 2
-#define RESCOL 4	/* columns reserved for row numbers */
 #define RESROW 3 /* rows reserved for prompt, error, and column numbers */
 
 /* formats for engformat() */
@@ -36,21 +35,18 @@
 #define DEFPREC   2
 #define DEFREFMT  REFMTFIX /* Make default format fixed point  THA 10/14/90 */
 
+#define FKEYS  24	/* Number of function keys available */
 #define HISTLEN  10	/* Number of history entries for vi emulation */
 #ifdef PSC
 # define error(msg)	fprintf(stderr, msg);
 #else
-# define error (void)move(1,0), (void)clrtoeol(), (void) printw
+# define error isatty(fileno(stdin)) && !move(1,0) && !clrtoeol() && printw
 #endif
 #define	FBUFLEN	1024	/* buffer size for a single field */
 #define	PATHLEN	1024	/* maximum path length */
 
 #ifndef A_CHARTEXT	/* Should be defined in curses.h */
-# ifdef INTERNATIONAL
-#  define A_CHARTEXT 0xff
-# else
-#  define A_CHARTEXT 0x7f
-# endif
+#define A_CHARTEXT 0xff
 #endif
 
 #if (defined(BSD42) || defined(BSD43)) && !defined(strrchr)
@@ -85,7 +81,7 @@ struct ent_ptr {
 
 /* holds the beginning/ending cells of a range */
 struct range_s {
-	struct ent_ptr left, right;
+    struct ent_ptr left, right;
 };
 
 /*
@@ -105,14 +101,16 @@ struct range_s {
 
 /* info for each cell, only alloc'd when something is stored in a cell */
 struct ent {
-    double v;		/* v && label are set in EvalAll() */
+    double v;			/* v && label are set in EvalAll() */
     char *label;
-    struct enode *expr;	/* cell's contents */
+    struct enode *expr;		/* cell's contents */
     short flags;	
     short row, col;
-    struct ent *next;	/* next deleted ent (pulled, deleted cells) */
-    char *format;	/* printf format for this cell */
-    char cellerror;	/* error in a cell? */
+    short nrow, ncol;		/* link to note */
+    short nlastrow, nlastcol;
+    struct ent *next;		/* next deleted ent (pulled, deleted cells) */
+    char *format;		/* printf format for this cell */
+    char cellerror;		/* error in a cell? */
 };
 
 /* stores a range (left, right) */
@@ -126,6 +124,12 @@ struct range {
 #define FIX_ROW 1
 #define FIX_COL 2
 
+struct frange {
+    struct ent *or_left, *or_right;	/* outer range */
+    struct ent *ir_left, *ir_right;	/* inner range */
+    struct frange *r_next, *r_prev;	/* chained ranges */
+};
+
 /* stores type of operation this cell will perform */
 struct enode {
     int op;
@@ -142,18 +146,19 @@ struct enode {
 };
 
 /* Use this structure to save the last 'g' command */
-extern	struct go_save {
-	int g_type;
-	double g_n;
-	char *g_s;
-	int  g_row;
-	int  g_col;
-	int  g_lastrow;
-	int  g_lastcol;
-	int  strow;
-	int  stcol;
-	int  errsearch;
-} gs;
+struct go_save {
+    int		g_type;
+    double	g_n;
+    char	*g_s;
+    int		g_row;
+    int		g_col;
+    int		g_lastrow;
+    int		g_lastcol;
+    int		strow;
+    int		stcol;
+    int		stflag;
+    int		errsearch;
+};
 
 /* op values */
 #define O_VAR 'v'
@@ -168,8 +173,8 @@ extern	struct go_save {
 #define ATAN (OP_BASE + 2)
 #define CEIL (OP_BASE + 3)
 #define COS (OP_BASE + 4)
-#define EXP (OP_BASE + 5 )
-#define FABS (OP_BASE + 6 )
+#define EXP (OP_BASE + 5)
+#define FABS (OP_BASE + 6)
 #define FLOOR (OP_BASE + 7)
 #define HYPOT (OP_BASE + 8)
 #define LOG (OP_BASE + 9)
@@ -210,7 +215,7 @@ extern	struct go_save {
 #define INDEX (OP_BASE + 44)
 #define DTS (OP_BASE + 45)
 #define TTS (OP_BASE + 46)
-#define ABS (OP_BASE + 47 )
+#define ABS (OP_BASE + 47)
 #define HLOOKUP (OP_BASE + 48)
 #define VLOOKUP (OP_BASE + 49)
 #define ROUND (OP_BASE + 50)
@@ -265,10 +270,12 @@ extern	struct go_save {
 extern	struct ent ***tbl;	/* data table ref. in vmtbl.c and ATBL() */
 
 extern	char curfile[];
+extern	int arg;
 extern	int strow, stcol;
 extern	int currow, curcol;
-extern	int savedrow[26], savedcol[26];
-extern	int savedstrow[26], savedstcol[26];
+extern	int rescol;		/* columns reserved for row numbers */
+extern	int savedrow[27], savedcol[27];
+extern	int savedstrow[27], savedstcol[27];
 extern	int FullUpdate;
 extern	int maxrow, maxcol;
 extern	int maxrows, maxcols;	/* # cells currently allocated */
@@ -286,7 +293,7 @@ extern	struct ent *to_fix;
 extern	int showsc, showsr;
 extern	int cellassign;
 
-extern	FILE *openout();
+extern	FILE *openfile(char *fname, int *rpid, int *rfd);
 extern	char *coltoa(int col);
 extern	char *findhome(char *path);
 extern	char *r_name(int r1, int c1, int r2, int c2);
@@ -299,7 +306,7 @@ extern	int are_ranges();
 extern	int atocol(char *string, int len);
 extern	int cwritefile(char *fname, int r0, int c0, int rn, int cn);
 extern	int engformat(int fmt, int width, int lprecision, double val,
-    char *buf, int buflen);
+	char *buf, int buflen);
 extern	int etype(register struct enode *e);
 extern	int format(char *fmt, double val, char *buf, int buflen);
 extern	int get_rcqual(int ch);
@@ -316,25 +323,26 @@ extern	struct enode *new_range(int op, struct range_s a1);
 extern	struct enode *new_str(char *s);
 extern	struct enode *new_var(int op, struct ent_ptr a1);
 extern	struct ent *lookat(int row, int col);
-extern	struct range *find_range(char *name, int len, struct ent *lmatch,
-    struct ent *rmatch);
+extern	int find_range(char *name, int len, struct ent *lmatch,
+	struct ent *rmatch, struct range **rng);
 extern	void EvalAll();
 extern	void add_range(char *name, struct ent_ptr left, struct ent_ptr right,
-    int is_range);
+	int is_range);
 extern	void backcol(int arg);
 extern	void backrow(int arg);
 extern	void checkbounds(int *rowp, int *colp);
 extern	void clearent(struct ent *v);
 extern	void clean_range();
 extern	void closecol(int cs, int numcol);
-extern	void closeout(FILE *f, int pid);
-extern	void closerow(register r);
+extern	void closefile(FILE *f, int pid, int rfd);
+extern	void closerow(int r, int numrow);
 extern	void colshow_op();
 extern	void copy(struct ent *dv1, struct ent *dv2, struct ent *v1,
-    struct ent *v2);
+	struct ent *v2);
 extern	void copyent(register struct ent *n, register struct ent *p, int dr,
-    int dc);
+	int dc);
 extern	void creadfile(char *save, int  eraseflg);
+extern	void decompile(register struct enode *e, int priority);
 extern	void deleterow(register int arg);
 extern	void del_range(struct ent *left, struct ent *right);
 extern	void deraw();
@@ -352,12 +360,19 @@ extern	void efree(struct enode *e);
 extern	void erase_area(int sr, int sc, int er, int ec);
 extern	void erasedb();
 extern	void eraser(struct ent *v1, struct ent *v2);
+extern	void fgetnum(int r0, int c0, int rn, int cn, int fd);
 extern	void fill(struct ent *v1, struct ent *v2, double start, double inc);
+extern	struct frange *find_frange(int row, int col);
 extern	void flush_saved();
 extern	void format_cell(struct ent *v1, struct ent *v2, char *s);
 extern	void forwcol(int arg);
 extern	void forwrow(int arg);
 extern	void free_ent(register struct ent *p);
+extern	void getexp(int r0, int c0, int rn, int cn, int fd);
+extern	void getfmt(int r0, int c0, int rn, int cn, int fd);
+extern	void getformat(int col, int fd);
+extern	void getnum(int r0, int c0, int rn, int cn, int fd);
+extern	void getstring(int r0, int c0, int rn, int cn, int fd);
 extern	void go_last();
 extern	void goraw();
 extern	void help();
@@ -366,18 +381,20 @@ extern	void hide_row(int arg);
 extern	void hidecol(int arg);
 extern	void hiderow(int arg);
 extern	void initkbd();
+extern	void ins_in_line(int c);
 extern	void ins_string(char *s);
 extern	void insert_mode();
-extern	void insertrow(register int arg);
 extern	void kbd_again();
 extern	void label(register struct ent *v, register char *s, int flushdir);
 extern	void let(struct ent *v, struct enode *e);
 extern	void list_range(FILE *f);
 extern	void lock_cells(struct ent *v1, struct ent *v2);
-extern	void moveto(int row, int col, int lastrow, int lastcol, int cornrow,
-    int corncol);
-extern	void num_search(double n, int errsearch);
-extern	void opencol(int cs, int numcol);
+extern	void moveto(int row, int col, int lastrow, int lastcol,
+	int cornrow, int corncol);
+extern	void toggle_navigate_mode();
+extern	void num_search(double n, int firstrow, int firstcol, int lastrow,
+	int lastcol, int errsearch);
+extern	void insertcol(int arg);
 extern	void printfile(char *fname, int r0, int c0, int rn, int cn);
 extern	void pullcells(int to_insert);
 extern	void readfile(char *fname, int eraseflg);
@@ -391,20 +408,26 @@ extern	void showcol(int c1, int c2);
 extern	void showdr();
 extern	void showrow(int r1, int r2);
 extern	void showstring(char *string, int dirflush, int hasvalue, int row,
-    int col, int *nextcolp, int mxcol, int *fieldlenp, int r, int c);
+	int col, int *nextcolp, int mxcol, int *fieldlenp, int r, int c,
+	struct frange *fr, int frightcols, int flcols, int frcols);
 extern	void signals();
 extern	void slet(struct ent *v, struct enode *se, int flushdir);
 extern	void startshow();
-extern	void str_search(char *s);
+extern	void stopdisp();
+extern	void str_search(char *s, int firstrow, int firstcol, int lastrow,
+	int lastcol);
 extern	void sync_ranges();
 extern	void sync_refs();
 extern	void tblprintfile(char *fname, int r0, int c0, int rn, int cn);
 extern	void unlock_cells(struct ent *v1, struct ent *v2);
+extern	void update(int anychanged);
 extern	void valueize_area(int sr, int sc, int er, int ec);
 extern	void write_fd(register FILE *f, int r0, int c0, int rn, int cn);
 extern	void write_line(int c);
 extern	void write_range(FILE *f);
 extern	void yyerror(char *err);
+extern	int yylex();
+extern	int yyparse();
 #ifdef DOBACKUPS
 extern	int backup_file(char *path);
 #endif
@@ -414,6 +437,15 @@ extern	int modflg;
 extern	int Crypt;
 #endif
 extern	char *mdir;
+extern	char *fkey[FKEYS];
+extern	char *scext;
+extern	char *ascext;
+extern	char *tbl0ext;
+extern	char *tblext;
+extern	char *latexext;
+extern	char *slatexext;
+extern	char *texext;
+extern	int scrc;
 extern	double prescale;
 extern	int extfunc;
 extern	int propagation;
