@@ -1,7 +1,7 @@
 /*	SC	A Spreadsheet Calculator
  *
  *	One line vi emulation
- *	$Revision: 7.2 $
+ *	$Revision: 7.7 $
  */
 
 #include <sys/types.h>
@@ -28,16 +28,15 @@ static int  back_word();
 static void back_space();
 static void col_0();
 static void cr_line();
-static void delete_cmd();
+static void delete_cmd(int change);
 static void del_chars(register int first, register int last);
-static void del_in_line();
+static void del_in_line(int back_null);
 static void del_to_end();
 static void dotcmd();
-static int  find_char();
+static int  find_char(int n);
 static void for_hist();
 static int  for_line(int stop_null);
-static int  for_word(int stop_null);
-static void ins_in_line(int c);
+static int  for_word(int end_word, int stop_null);
 static void last_col();
 static void rep_char();
 static void replace_in_line(int c);
@@ -49,11 +48,12 @@ static void search_again();
 static void search_hist();
 static void search_mode();
 static void stop_edit();
-static int  to_char();
+static int  to_char(int n);
 static void u_save(int c);
 
 extern int showrange;
 extern char mode_ind;		/* Mode indicator */
+extern int lcols;		/* Spreadsheet Column the cursor was in last */
 
 /* values for mode below */
 
@@ -61,6 +61,8 @@ extern char mode_ind;		/* Mode indicator */
 #define EDIT_MODE       1	/* Edit mode */
 #define REP_MODE        2	/* Replace mode */
 #define SEARCH_MODE	3	/* Get arguments for '/' command */
+#define NAVIGATE_MODE	4	/* Navigate the spreadsheet while editing
+					a line */
 
 #define	DOTLEN		200
 
@@ -68,10 +70,10 @@ static int mode = INSERT_MODE;
 struct	hist {
 	unsigned int	len;
 	char		*histline;
-} history[HISTLEN];
+} history[HISTLEN + 1];
 
-static int	histp = -1;
-static int	lasthist = -1;
+static int	histp = 0;
+static int	lasthist = 0;
 static int	endhist = -1;
 static char	*last_search = NULL;
 static char	*undo_line = NULL;
@@ -79,58 +81,134 @@ static int	undo_lim;
 static char	dotb[DOTLEN];
 static int	doti = 0;
 static int	do_dot = 0;
+static char	putbuf[FBUFLEN];
 
 void
 write_line(int c)
 {
     if (mode == EDIT_MODE) {
-	switch(c) {
+	switch (c) {
+#ifdef KEY_BACKSPACE
+	case KEY_BACKSPACE:
+#endif
 	case (ctl('h')):	linelim = back_line();			break;
+	case (ctl('i')):	startshow();				break;
 	case (ctl('m')):	cr_line();				break;
+	case (ctl('v')):	toggle_navigate_mode();			break;
 	case ESC:	stop_edit();					break;
 	case '+':	for_hist();					break;
 	case '-':	back_hist();					break;
+#ifdef KEY_END
+	case KEY_END:
+#endif
 	case '$':	last_col();					break;
 	case '.':	dotcmd();					break;
+#ifdef KEY_FIND
+	case KEY_FIND:
+#endif
 	case '/':	search_mode();					break;
+#ifdef KEY_HOME
+	case KEY_HOME:
+#endif
 	case '0':	col_0();					break;
+	case 'A':	u_save(c); last_col(); append_line();		break;
 	case 'C':	u_save(c); del_to_end(); append_line();		break;
 	case 'D':	u_save(c); del_to_end();			break;
+	case 'F':	linelim = find_char(-1);			break;
+	case 'G':	if (histp > 0) histp = lasthist; for_hist();	break;
 	case 'I':	u_save(c); col_0(); insert_mode();		break;
+	case 'P':	u_save(c);
+			ins_string(putbuf);
+			linelim = back_line();				break;
 	case 'R':	replace_mode();					break;
+	case 'T':	linelim = to_char(-1);				break;
 	case 'X':	u_save(c); back_space();			break;
 	case 'a':	u_save(c); append_line();			break;
-	case 'A':	u_save(c); last_col(); append_line();		break;
 	case 'b':	linelim = back_word();				break;
-	case 'c':	u_save(c); delete_cmd(); insert_mode();		break;
-	case 'd':	u_save(c); delete_cmd();			break;
-	case 'f':	linelim = find_char();				break;
+	case 'c':	u_save(c); delete_cmd(1); insert_mode();	break;
+	case 'd':	u_save(c); delete_cmd(0);			break;
+	case 'e':	linelim = for_word(1, 0);			break;
+	case 'f':	linelim = find_char(1);				break;
+#ifdef KEY_LEFT
+	case KEY_LEFT:
+#endif
+	case (ctl('b')):
 	case 'h':	linelim = back_line();				break;
+#ifdef KEY_IC
+	case KEY_IC:
+#endif
 	case 'i':	u_save(c); insert_mode();			break;
+#ifdef KEY_DOWN
+	case KEY_DOWN:
+#endif
 	case 'j':	for_hist();					break;
+#ifdef KEY_UP
+	case KEY_UP:
+#endif
 	case 'k':	back_hist();					break;
+#ifdef KEY_RIGHT
+	case KEY_RIGHT:
+#endif
+	case (ctl('f')):
 	case ' ':
 	case 'l':	linelim = for_line(0);				break;
 	case 'n':	search_again();					break;
+	case 'p':	u_save(c);
+			linelim = for_line(1);
+			ins_string(putbuf);
+			linelim = back_line();				break;
 	case 'q':	stop_edit();					break;
 	case 'r':	u_save(c); rep_char();				break;
-	case 's':	u_save(c); del_in_line(); insert_mode();	break;
-	case 't':	linelim = to_char();				break;
+	case 's':	u_save(c); del_in_line(0); insert_mode();	break;
+	case 't':	linelim = to_char(1);				break;
 	case 'u':	restore_it();					break;
-	case 'w':	linelim = for_word(0);				break;
-	case 'x':	u_save(c); del_in_line();			break;
+	case 'w':	linelim = for_word(0, 0);			break;
+#ifdef KEY_DC
+	case KEY_DC:
+#endif
+	case 'x':	u_save(c); del_in_line(1);			break;
 	default:							break;
 	}
     } else if (mode == INSERT_MODE) { 
 	savedot(c);
-	switch(c) {
+	switch (c) {
+#ifdef KEY_BACKSPACE
+	case KEY_BACKSPACE:
+#endif
 	case (ctl('h')):	back_space();				break;
+	case (ctl('i')):	startshow();				break;
 	case (ctl('m')):	cr_line();				break;
+	case (ctl('v')):	toggle_navigate_mode();			break;
+#ifdef KEY_LEFT
+	case KEY_LEFT:
+#endif
+	case (ctl('b')):	linelim = back_line();			break;
+#ifdef KEY_RIGHT
+	case KEY_RIGHT:
+#endif
+	case (ctl('f')):	linelim = for_line(1);			break;
+#ifdef KEY_DOWN
+	case KEY_DOWN:
+#endif
+	case (ctl('n')):	for_hist();				break;
+#ifdef KEY_UP
+	case KEY_UP:
+#endif
+	case (ctl('p')):	back_hist();				break;
+#ifdef KEY_HOME
+	case KEY_HOME:		col_0();				break;
+#endif
+#ifdef KEY_END
+	case KEY_END:		last_col();				break;
+#endif
 	case ESC:		edit_mode();				break;
 	default:		ins_in_line(c);				break;
 	}
     } else if (mode == SEARCH_MODE) {
-	switch(c) {
+	switch (c) {
+#ifdef KEY_BACKSPACE
+	case KEY_BACKSPACE:
+#endif
 	case (ctl('h')):	back_space();				break;
 	case (ctl('m')):	search_hist();				break;
 	case ESC:		edit_mode();				break;
@@ -138,12 +216,174 @@ write_line(int c)
 	}
     } else if (mode == REP_MODE) {
 	savedot(c);
-	switch(c) {
+	switch (c) {
+#ifdef KEY_BACKSPACE
+	case KEY_BACKSPACE:
+#endif
 	case (ctl('h')):	back_space();				break;
 	case (ctl('m')):	cr_line();				break;
 	case ESC:		edit_mode();				break;
 	default:		replace_in_line(c);			break;
 	}
+    } else if (mode == NAVIGATE_MODE) {
+	switch (c) {
+	case ('.'):
+	case (ctl('i')):	if (!showrange) {
+				    toggle_navigate_mode();
+				    startshow();
+				} else {
+				    showdr();
+				}					break;
+	case (ctl('m')):	if (!showrange) {
+				    ins_string(v_name(currow, curcol));
+				    toggle_navigate_mode();
+				} else {
+				    toggle_navigate_mode();
+				    cr_line();
+				}					break;
+	case (ctl('v')):	ins_string(v_name(currow, curcol));
+				toggle_navigate_mode();			break;
+#ifdef KEY_LEFT
+	case KEY_LEFT:
+#endif
+	case (ctl('b')):
+	case 'h':		backcol(arg);				break;
+#ifdef KEY_RIGHT
+	case KEY_RIGHT:
+#endif
+	case (ctl('f')):
+	case ' ':
+	case 'l':		forwcol(arg);				break;
+#ifdef KEY_DOWN
+	case KEY_DOWN:
+#endif
+	case (ctl('n')):
+	case 'j':		forwrow(arg);				break;
+#ifdef KEY_UP
+	case KEY_UP:
+#endif
+	case (ctl('p')):
+	case 'k':		backrow(arg);				break;
+	case 'q':
+	case ctl('g'):
+	case ESC:		toggle_navigate_mode();
+				showrange = 0;				break;
+	case 'H':
+				backcol((curcol-stcol+1)+1);
+									break;
+#ifdef KEY_NPAGE
+	case KEY_NPAGE:			/* next page */
+#endif
+	case 'J':
+				forwrow((LINES-RESROW)/2);
+				strow = strow + (LINES-RESROW)/2;
+				FullUpdate++;
+									break;
+#ifdef	KEY_PPAGE
+	case KEY_PPAGE:			/* previous page */
+#endif
+	case 'K':
+				backrow((LINES-RESROW)/2);
+				if (strow > (LINES-RESROW)/2)
+				    strow = strow - (LINES-RESROW)/2;
+				else
+				    strow = 0;
+				FullUpdate++;
+									break;
+	case 'L':
+				forwcol(lcols -(curcol-stcol)+1);
+									break;
+#ifdef KEY_HOME
+	case KEY_HOME:
+				currow = 0;
+				curcol = 0;
+				rowsinrange = 1;
+				colsinrange = fwidth[curcol];
+				FullUpdate++;
+									break;
+#endif
+	case '0':		curcol = 0;
+		   		rowsinrange = 1;
+		   		colsinrange = fwidth[curcol];		break;
+	case '$':
+				{
+				register struct ent *p;
+
+				curcol = maxcols - 1;
+				while (!VALID_CELL(p, currow, curcol) &&
+					curcol > 0)
+				    curcol--;
+				rowsinrange = 1;
+				colsinrange = fwidth[curcol];
+				break;
+				}
+	case '^':
+				currow = 0;
+				rowsinrange = 1;
+				colsinrange = fwidth[curcol];		break;
+	case '#':
+				{
+				register struct ent *p;
+
+				currow = maxrows - 1;
+				while (!VALID_CELL(p, currow, curcol) &&
+					currow > 0)
+				    currow--;
+				rowsinrange = 1;
+				colsinrange = fwidth[curcol];		break;
+				}
+	case 'w':
+				{
+				register struct ent *p;
+
+				while (--arg>=0) {
+				    do {
+					if (curcol < maxcols - 1)
+					    curcol++;
+					else {
+					    if (currow < maxrows - 1) {
+						while(++currow < maxrows - 1 &&
+							row_hidden[currow])
+						    ;
+						curcol = 0;
+					    } else {
+						error("At end of table");
+						break;
+					    }
+					}
+				    } while(col_hidden[curcol] ||
+					    !VALID_CELL(p, currow, curcol));
+				}
+				rowsinrange = 1;
+				colsinrange = fwidth[curcol];
+				break;
+				}
+	case 'b':
+				{
+				register struct ent *p;
+
+				while (--arg>=0) {
+				    do {
+					if (curcol) 
+					    curcol--;
+					else {
+					    if (currow) {
+						while(--currow && row_hidden[currow])
+						    ;
+						curcol = maxcols - 1;
+					    } else {
+						error("At start of table");
+						break;
+					    }
+					}
+				    } while (col_hidden[curcol] ||
+					    !VALID_CELL(p, currow, curcol));
+				}
+				rowsinrange = 1;
+				colsinrange = fwidth[curcol];
+				break;
+				}
+	}
     }
 }
 
@@ -152,7 +392,6 @@ edit_mode()
 {
     mode = EDIT_MODE;
     mode_ind = 'e';
-    histp = -1;
     if (linelim < 0)	/* -1 says stop editing, ...so we still aren't */
 	return;
     linelim = back_line();
@@ -171,7 +410,6 @@ search_mode()
     line[0] = '/';
     line[1] = '\0';
     linelim = 1;
-    histp = -1;
     mode_ind = '/';
     mode = SEARCH_MODE;
 }
@@ -183,6 +421,60 @@ replace_mode()
     mode = REP_MODE;
 }
 
+void
+toggle_navigate_mode()
+{
+    static char prev_mode = NAVIGATE_MODE;
+    int limtmp;
+
+    switch (prev_mode) {
+	case INSERT_MODE:
+				prev_mode = NAVIGATE_MODE;
+				insert_mode();
+				break;
+	case EDIT_MODE:
+				prev_mode = NAVIGATE_MODE;
+				limtmp = linelim;
+				edit_mode();
+				linelim = limtmp;
+				break;
+	case NAVIGATE_MODE:
+				prev_mode = mode;
+				mode_ind = 'v';
+				mode = NAVIGATE_MODE;
+				break;
+	default:
+				prev_mode = NAVIGATE_MODE;
+				break;
+    }
+}
+
+/* show the current range (see ^I), we are moving around to define a range */
+void
+startshow()
+{
+    showrange = 1;
+    showsr = currow;
+    showsc = curcol;
+    toggle_navigate_mode();
+}
+
+/* insert the range we defined by moving around the screen, see startshow() */
+void
+showdr()
+{
+    int		minsr, minsc, maxsr, maxsc;
+    char	*p;
+
+    minsr = showsr < currow ? showsr : currow;
+    minsc = showsc < curcol ? showsc : curcol;
+    maxsr = showsr > currow ? showsr : currow;
+    maxsc = showsc > curcol ? showsc : curcol;
+    toggle_navigate_mode();
+    ins_string(r_name(minsr, minsc, maxsr, maxsc));
+    showrange = 0;
+}
+
 /* dot command functions.  Saves info so we can redo on a '.' command */
 
 static void
@@ -191,8 +483,14 @@ savedot(int c)
     if (do_dot || (c == '\n'))
 	return;
 
-    if (doti < DOTLEN-1)
-    {
+    if (doti < DOTLEN-1) {
+	if (c > 255) {
+	    if (doti < DOTLEN-2) {
+		dotb[doti++] = c/256;
+		c %= 256;
+	    } else
+		return;
+	}
 	dotb[doti++] = c;
 	dotb[doti] = '\0';
     }
@@ -209,8 +507,9 @@ dotcmd()
 	return;
     do_dot = 1;
     doti = 0;
-    while(dotb[doti] != '\0') {
-	c = dotb[doti++];
+    while (dotb[doti] != '\0') {
+	if ((c = dotb[doti++]) < 4)
+	    c = c * 256 + dotb[doti++];
 	dotcalled = 1;
 	write_line(c);
     }
@@ -224,18 +523,18 @@ vigetch()
 {
     int c;
 
-    if(do_dot) {
+    if (do_dot) {
 	if (dotb[doti] != '\0') {
-	    return(dotb[doti++]);
+	    return (dotb[doti++]);
 	} else {
 	    do_dot = 0;
 	    doti = 0;
-	    return(nmgetch());
+	    return (nmgetch());
 	}
     }
     c = nmgetch();
     savedot(c);
-    return(c);
+    return (c);
 }
 
 /* saves the current line for possible use by an undo cmd */
@@ -244,8 +543,8 @@ u_save(int c)
 {
     static unsigned	undolen = 0;
 
-    if (strlen(line)+1 > undolen)
-    {	undolen = strlen(line)+40;
+    if (strlen(line)+1 > undolen) {
+    	undolen = strlen(line)+40;
 
 	undo_line = scxrealloc(undo_line, undolen);
     }
@@ -272,8 +571,8 @@ restore_it()
     if ((undo_line == NULL) || (*undo_line == '\0')) 
 	return;
 
-    if (strlen(line)+1 > templen)
-    {	templen = strlen(line)+40;
+    if (strlen(line)+1 > templen) {
+    	templen = strlen(line) + 40;
 	tempc = scxrealloc(tempc, templen);
     }
 
@@ -306,37 +605,47 @@ static int
 for_line(int stop_null)
 {
     if (linelim >= 0 && line[linelim] != '\0' && 
-    		        (line[linelim+1] != '\0' || stop_null))
-	return(linelim+1);
+	    (line[linelim+1] != '\0' || stop_null))
+	return (linelim+1);
     else
-	return(linelim);
+	return (linelim);
 }
 
+/* If end_word is non-zero, go to next end-of-word.  Otherwise, go to next
+ * beginning-of-word.
+ */
+
 static int
-for_word(int stop_null)
+for_word(int end_word, int stop_null)
 {
     register int c;
     register int cpos;
 
     cpos = linelim;
 
+    if (end_word)
+	cpos++;
+
     if (line[cpos] == ' ') {
 	while (line[cpos] == ' ')
 	    cpos++;
 	if (cpos > 0 && line[cpos] == '\0')
 	    --cpos;
-	return(cpos);
+	if (!end_word)
+	    return(cpos);
     }
 
     if (istext(line[cpos])) {
     	while ((c = line[cpos]) && istext(c)) 
-		cpos++;
+	    cpos++;
     } else {
 	while ((c = line[cpos]) && !istext(c) && c != ' ')
-		cpos++;
+	    cpos++;
     }
 
-    while (line[cpos] == ' ')
+    if (end_word)
+	cpos--;
+    else while (line[cpos] == ' ')
         cpos++;
 
     if (cpos > 0 && line[cpos] == '\0' && !stop_null) 
@@ -393,8 +702,11 @@ back_word()
 
 /* Text manipulation commands */
 
+/* If back_null is set, back up if the deletion leaves you at the null
+ * line terminator.  Otherwise, don't.
+ */
 static void
-del_in_line()
+del_in_line(int back_null)
 {
     register int len, i;
 
@@ -402,27 +714,31 @@ del_in_line()
 	len = strlen(line);
 	if (linelim == len && linelim > 0)
 	    linelim--;
+	putbuf[0] = line[linelim];
+	putbuf[1] = '\0';
 	for (i = linelim; i < len; i++)
 	    line[i] = line[i+1];
     }
-    if (linelim > 0 && line[linelim] == '\0')
+    if (back_null && linelim > 0 && line[linelim] == '\0')
 	--linelim;
 }
 
-static void
+void
 ins_in_line(int c)
 {
     register int i, len;
 
-    if (linelim < 0)
-    {	*line = '\0';
-	linelim = 0;
+    if (c < 256) {
+	if (linelim < 0) {
+	    *line = '\0';
+	    linelim = 0;
+	}
+	len = strlen(line);
+	for (i = len; i >= linelim; --i)
+	    line[i+1] = line[i];
+	line[linelim++] = c;
+	line[len+1] = '\0';
     }
-    len = strlen(line);
-    for (i = len; i >= linelim; --i)
-	line[i+1] = line[i];
-    line[linelim++] = c;
-    line[len+1] = '\0';
 }
 
 void
@@ -448,16 +764,15 @@ rep_char()
 {
     int c;
 
-    if (linelim < 0)
-    {	linelim = 0;
+    if (linelim < 0) {
+    	linelim = 0;
 	*line = '\0';
     }
     c = vigetch();
-    if (line[linelim] != '\0') {
-    	line[linelim] = c;
-    } else {
+    if (c < 256 && c!= ESC && c != ctl('g')) {
+	if (line[linelim] == '\0')
+	    line[linelim+1] = '\0';
 	line[linelim] = c;
-	line[linelim+1] = '\0';
     }
 }
 
@@ -466,14 +781,16 @@ replace_in_line(int c)
 {
     register int len;
 
-    if (linelim < 0)
-    {	linelim = 0;
-	*line = '\0';
+    if (c < 256) {
+	if (linelim < 0) {
+	    linelim = 0;
+	    *line = '\0';
+	}
+	len = strlen(line);
+	line[linelim++] = c;
+	if (linelim > len)
+	    line[linelim] = '\0';
     }
-    len = strlen(line);
-    line[linelim++] = c;
-    if (linelim > len)
-	line[linelim] = '\0';
 }
 
 static void
@@ -484,37 +801,51 @@ back_space()
 
     if (line[linelim] == '\0') {
 	linelim = back_line();
-	del_in_line();
+	del_in_line(1);
 	linelim = strlen(line);
     } else {
 	linelim = back_line();
-	del_in_line();
+	del_in_line(1);
     }
 }
 
+/* Setting change to 1 makes `w' act like `e' so that `cw' will act like
+ * `ce', just as in vi.  Setting change to 0 causes `w' to act as usual.
+ */
+
 int
-get_motion()
+get_motion(int change)
 {
     int c;
 
     c = vigetch();
     switch (c) {
-    case 'b':	return(back_word());
-    case 'f':	return(find_char()+1);
-    case 'h':	return(back_line());
-    case 'l':	return(for_line(1));
-    case 't':	return(to_char()+1);
-    case 'w':	return(for_word(1));
-    default:	return(linelim);
+	case '0':	return (0);
+	case '$':	return (strlen(line));
+	case 'b':	return (back_word());
+	case 'c':	return (change ? -1 : linelim);
+	case 'd':	return (!change ? -1 : linelim);
+	case 'e':	return (for_word(1, 1) + 1);
+	case 'f':	return ((c = find_char(1)) == linelim ? c : c + 1);
+	case 'F':	return (find_char(-1));
+	case 'h':	return (back_line());
+	case 'l':	return (for_line(1));
+	case 't':	return ((c = to_char(1)) == linelim ? c : c + 1);
+	case 'T':	return (to_char(-1));
+	case 'w':	return (for_word(change, 1) + change);
+	default:	return (linelim);
     }
 }
 
 static void
-delete_cmd()
+delete_cmd(int change)
 {
     int cpos;
 
-    cpos = get_motion();
+    if ((cpos = get_motion(change)) == -1) {
+	cpos++;
+	linelim = strlen(line);
+    }
     del_chars(cpos, linelim);
 }
 
@@ -531,10 +862,9 @@ del_chars(register int first, register int last)
     }
 
     linelim = first;
-    while(first < last) {
-	del_in_line();
-	--last;
-    }
+    strncpy(putbuf, line + first, last - first);
+    putbuf[last - first] = '\0';
+    memmove(line + first, line + last, strlen(line + last) + 1);
 }
 
 static void
@@ -542,6 +872,7 @@ del_to_end()
 {
     if (linelim < 0)
 	return;
+    strcpy(putbuf, line + linelim);
     line[linelim] = '\0';
     linelim = back_line();
 }
@@ -563,8 +894,7 @@ cr_line()
 		    if ((rowlimit >= 0) && (currow >= rowlimit)) {
 			forwcol(1);
 			currow = 0;
-		    }
-		    else {
+		    } else {
 			forwrow(1);
 		    }
 		    break;
@@ -572,8 +902,7 @@ cr_line()
 		    if ((collimit >= 0) && (curcol >= collimit)) {
 			forwrow(1);
 			curcol = 0;
-		    }
-		    else {
+		    } else {
 			forwcol(1);
 		    }
 		    break;
@@ -582,8 +911,8 @@ cr_line()
 	    }
 	}
     }
-    else	/* '\n' alone will put you into insert mode */
-    {	*line = '\0';
+    else {	/* '\n' alone will put you into insert mode */
+    	*line = '\0';
 	linelim = 0;
     }
 }
@@ -594,39 +923,47 @@ static void
 save_hist()
 {
     if (lasthist < 0)
-    {	lasthist = 0;
-    }
+	lasthist = 1;
     else
-	lasthist = (lasthist + 1) % HISTLEN;
+	lasthist = lasthist % HISTLEN + 1;
 
     if (lasthist > endhist)
 	endhist = lasthist;
 
-    if (history[lasthist].len < strlen(line)+1)
-    {	history[lasthist].len = strlen(line)+40;
+    if (history[lasthist].len < strlen(line)+1) {
+    	history[lasthist].len = strlen(line)+40;
 	history[lasthist].histline = scxrealloc(history[lasthist].histline,
-					      history[lasthist].len);
+		history[lasthist].len);
     }
     (void) strcpy(history[lasthist].histline, line);
+    if (history[0].histline) {
+	scxfree(history[0].histline);
+	history[0].histline = (void *)0;
+	history[0].len = 0;
+    }
+    histp = 0;
 }
 
 static void
 back_hist()
 {
-    if (histp == -1)
-	histp = lasthist;
-    else
-    if (histp == 0)
-    {	if (endhist != lasthist)
-		histp = endhist;
-    }
-    else
-    if (histp != ((lasthist + 1) % (endhist + 1)))
+    if (histp == 0) {
+	if (history[0].len < strlen(line) + 1) {
+	    history[0].len = strlen(line) + 40;
+	    history[0].histline = scxrealloc(history[0].histline,
+		    history[0].len);
+	}
+	(void) strcpy(history[0].histline, line);
+
+	if (lasthist >= 0)
+	    histp = lasthist;
+    } else if (histp == 1) {
+    	if (endhist != lasthist)
+	    histp = endhist;
+    } else if (histp != ((lasthist + 1) % (endhist + 1)))
 	histp--;
 
-    if (lasthist < 0)
-	line[linelim = 0] = '\0';
-    else {
+    if (lasthist >= 0) {
     	(void) strcpy(line, history[histp].histline);
 	linelim = 0;
     }
@@ -643,8 +980,8 @@ search_hist()
 	return;
     }
 
-    if (strlen(line)+1 > lastsrchlen)
-    {	lastsrchlen = strlen(line)+40;
+    if (strlen(line)+1 > lastsrchlen) {
+    	lastsrchlen = strlen(line)+40;
 	last_search = scxrealloc(last_search, lastsrchlen);
     }
     (void)strcpy(last_search, line+1);
@@ -687,15 +1024,15 @@ search_again()
 static void
 for_hist()
 {
-    if (histp == -1)
-	histp = lasthist;
-    else
-    if (histp != lasthist)
-	histp = (histp + 1) % (endhist + 1);
+    if (histp == 0)
+    	return;
 
-    if (lasthist < 0)
-	line[linelim = 0] = '\0';
-    else {
+    if (histp == lasthist)
+	histp = 0;
+    else
+	histp = histp % endhist + 1;
+
+    if (lasthist >= 0) {
 	(void) strcpy(line, history[histp].histline);
 	linelim = 0;
     }
@@ -711,12 +1048,12 @@ static void
 last_col()
 {
     linelim = strlen(line);
-    if (linelim > 0)
+    if (linelim > 0 && mode_ind == 'e')
 	--linelim;
 }
 
 static int
-find_char()
+find_char(int n)
 {
     register int c;
     register int i;
@@ -724,21 +1061,21 @@ find_char()
 
     c = vigetch();
     i = linelim;
-    while(line[i] && line[i] != c)
-	i++;
-    if (!line[i])
+    while (i >= 0 && line[i] && line[i] != c)
+	i += n;
+    if (i < 0 || !line[i])
 	i = linelim;
-    return(i);
+    return (i);
 }
 
 static int
-to_char()
+to_char(int n)
 {
     register int i;
 
-    i = find_char();
-    if (i > 0 && i != linelim)
-	--i;
+    i = find_char(n);
+    if (i != linelim)
+	i -= n;
 
-    return(i);
+    return (i);
 }
