@@ -7,7 +7,7 @@
  *
  *              More mods Robert Bond, 12/86
  *		More mods by Alan Silverstein, 3-4/88, see list of changes.
- *		$Revision: 7.2 $
+ *		$Revision: 7.7 $
  *
  */
 #ifdef PSC
@@ -31,13 +31,13 @@ checkbounds(int *rowp, int *colp)
     else if (*rowp >= maxrows) {
 	if (*colp >= maxcols) {
 	    if (!growtbl(GROWBOTH, *rowp, *colp)) {
-		*rowp = maxrows -1;
-		*colp = maxcols -1;
+		*rowp = maxrows - 1;
+		*colp = maxcols - 1;
 	    }
 	    return;
 	} else {
 	    if (!growtbl(GROWROW, *rowp, 0))
-		*rowp = maxrows-1;
+		*rowp = maxrows - 1;
 	    return;
 	}
     }
@@ -45,7 +45,7 @@ checkbounds(int *rowp, int *colp)
 	*colp = 0;
     else if (*colp >= maxcols) {
 	if (!growtbl(GROWCOL, 0, *colp))
-	    *colp = maxcols-1;
+	    *colp = maxcols - 1;
     }
 }
 #endif /* !PSC */
@@ -53,10 +53,10 @@ checkbounds(int *rowp, int *colp)
 /* scxrealloc will just scxmalloc if oldptr is == NULL */
 #define GROWALLOC(newptr, oldptr, nelem, type, msg) \
     newptr = (type *)scxrealloc((char *)oldptr, \
-	(unsigned)(nelem*sizeof(type))); \
+	    (unsigned)(nelem * sizeof(type))); \
     if (newptr == (type *)NULL) { \
 	error(msg); \
-	return(FALSE); \
+	return (FALSE); \
     } \
     oldptr = newptr /* wait incase we can't alloc */
 
@@ -97,7 +97,7 @@ growtbl(int rowcol, int toprow, int topcol)
 	{   int startval;
 	    startval = LINES - RESROW;
 	    newrows = startval > MINROWS ? startval : MINROWS;
-	    startval = ((COLS) - RESCOL) / DEFWIDTH;
+	    startval = ((COLS) - rescol) / DEFWIDTH;
 	    newcols = startval > MINCOLS ? startval : MINCOLS;
 	}
 #else
@@ -118,7 +118,7 @@ growtbl(int rowcol, int toprow, int topcol)
 	if ((rowcol == GROWCOL) && ((maxcols == ABSMAXCOLS) ||
 		(topcol >= ABSMAXCOLS))) {
 	    error(nowider);
-	    return(FALSE);
+	    return (FALSE);
 	}
 
 	if (topcol > maxcols)
@@ -165,7 +165,7 @@ growtbl(int rowcol, int toprow, int topcol)
 	for (i = maxcols; i < newcols; i++) {
 	    fwidth[i] = DEFWIDTH;
 	    precision[i] = DEFPREC;
-	    realfmt[i]= DEFREFMT;
+	    realfmt[i] = DEFREFMT;
 	}
 
 	/* [re]alloc the space for each row */
@@ -194,12 +194,15 @@ growtbl(int rowcol, int toprow, int topcol)
 	    return(FALSE);
 	}
 	for (nullit = tbl[i], cnt = 0; cnt < newcols; cnt++, nullit++)
-		*nullit = (struct ent *)NULL;
+	    *nullit = (struct ent *)NULL;
 /*	memset((char *)tbl[i], 0, newcols*sizeof(struct ent **));*/
     }
 
     FullUpdate++;
     maxrows = newrows;
+
+    if (maxrows > 1000) rescol = 5;
+    if (maxrows > 10000) rescol = 6;
 #endif /* PSC */
 
     maxcols = newcols;
